# sage_setup: distribution = sagemath-graphs
# sage.doctest: needs sage.graphs
r"""
Finite Delta-complexes

AUTHORS:

- John H. Palmieri (2009-08)

This module implements the basic structure of finite
`\Delta`-complexes.  For full mathematical details, see Hatcher [Hat2002]_,
especially Section 2.1 and the Appendix on "Simplicial CW Structures".
As Hatcher points out, `\Delta`-complexes were first introduced by Eilenberg
and Zilber [EZ1950]_, although they called them "semi-simplicial complexes".

A `\Delta`-complex is a generalization of a :mod:`simplicial complex
<sage.homology.simplicial_complex>`; a `\Delta`-complex `X` consists
of sets `X_n` for each nonnegative integer `n`, the elements of which
are called *n-simplices*, along with *face maps* between these sets of
simplices: for each `n` and for all `0 \leq i \leq n`, there are
functions `d_i` from `X_n` to `X_{n-1}`, with `d_i(s)` equal to the
`i`-th face of `s` for each simplex `s \in X_n`.  These maps must
satisfy the *simplicial identity*

  .. MATH::

    d_i d_j = d_{j-1} d_i \text{ for all } i<j.

Given a `\Delta`-complex, it has a *geometric realization*: a
topological space built by taking one topological `n`-simplex for each
element of `X_n`, and gluing them together as determined by the face
maps.

`\Delta`-complexes are an alternative to simplicial complexes.  Every
simplicial complex is automatically a `\Delta`-complex; in the other
direction, though, it seems in practice that one can often construct
`\Delta`-complex representations for spaces with many fewer simplices
than in a simplicial complex representation.  For example, the minimal
triangulation of a torus as a simplicial complex contains 14
triangles, 21 edges, and 7 vertices, while there is a `\Delta`-complex
representation of a torus using only 2 triangles, 3 edges, and 1
vertex.

.. NOTE::

   This class derives from
   :class:`~sage.homology.cell_complex.GenericCellComplex`, and so
   inherits its methods.  Some of those methods are not listed here;
   see the :mod:`Generic Cell Complex <sage.homology.cell_complex>`
   page instead.
"""

from copy import copy
from sage.topology.cell_complex import GenericCellComplex
from sage.rings.integer_ring import ZZ
from sage.rings.rational_field import QQ
from sage.rings.integer import Integer
from .simplicial_complex import Simplex, lattice_paths, SimplicialComplex
from sage.arith.misc import binomial
from sage.misc.cachefunc import cached_method
from sage.misc.lazy_import import lazy_import

lazy_import('sage.matrix.constructor', 'matrix')


class DeltaComplex(GenericCellComplex):
    r"""
    Define a `\Delta`-complex.

    INPUT:

    - ``data`` -- see below for a description of the options
    - ``check_validity`` -- boolean (default: ``True``); if ``True``, check
      that the simplicial identities hold

    OUTPUT: a `\Delta`-complex

    Use ``data`` to define a `\Delta`-complex.  It may be in any of
    three forms:

    - ``data`` may be a dictionary indexed by simplices.  The value
      associated to a d-simplex `S` can be any of:

      - a list or tuple of (d-1)-simplices, where the i-th entry is the
        i-th face of S, given as a simplex,

      - another d-simplex `T`, in which case the i-th face of `S` is
        declared to be the same as the i-th face of `T`: `S` and `T`
        are glued along their entire boundary,

      - ``None`` or ``True`` or ``False`` or anything other than the previous two
        options, in which case the faces are just the ordinary faces of `S`.

      For example, consider the following::

        sage: n = 5
        sage: S5 = DeltaComplex({Simplex(n):True, Simplex(range(1,n+2)): Simplex(n)})
        sage: S5
        Delta complex with 6 vertices and 65 simplices

      The first entry in dictionary forming the argument to
      ``DeltaComplex`` says that there is an `n`-dimensional simplex
      with its ordinary boundary.  The second entry says that there is
      another simplex whose boundary is glued to that of the first
      one.  The resulting `\Delta`-complex is, of course, homeomorphic
      to an `n`-sphere, or actually a 5-sphere, since we defined `n`
      to be 5.  (Note that the second simplex here can be any
      `n`-dimensional simplex, as long as it is distinct from
      ``Simplex(n)``.)

      Let's compute its homology, and also compare it to the simplicial version::

        sage: S5.homology()                                                             # needs sage.modules
        {0: 0, 1: 0, 2: 0, 3: 0, 4: 0, 5: Z}
        sage: S5.f_vector()  # number of simplices in each dimension
        [1, 6, 15, 20, 15, 6, 2]
        sage: simplicial_complexes.Sphere(5).f_vector()
        [1, 7, 21, 35, 35, 21, 7]

      Both contain a single (-1)-simplex, the empty simplex; other
      than that, the `\Delta`-complex version contains fewer simplices
      than the simplicial one in each dimension.

      To construct a torus, use::

        sage: torus_dict = {Simplex([0,1,2]): True,
        ....:        Simplex([3,4,5]): (Simplex([0,1]), Simplex([0,2]), Simplex([1,2])),
        ....:        Simplex([0,1]): (Simplex(0), Simplex(0)),
        ....:        Simplex([0,2]): (Simplex(0), Simplex(0)),
        ....:        Simplex([1,2]): (Simplex(0), Simplex(0)),
        ....:        Simplex(0): ()}
        sage: T = DeltaComplex(torus_dict); T
        Delta complex with 1 vertex and 7 simplices
        sage: T.cohomology(base_ring=QQ)                                                # needs sage.modules
        {0: Vector space of dimension 0 over Rational Field,
         1: Vector space of dimension 2 over Rational Field,
         2: Vector space of dimension 1 over Rational Field}

      This `\Delta`-complex consists of two triangles (given by
      ``Simplex([0,1,2])`` and ``Simplex([3,4,5])``); the boundary of
      the first is just its usual boundary: the 0th face is obtained
      by omitting the lowest numbered vertex, etc., and so the
      boundary consists of the edges ``[1,2]``, ``[0,2]``, and
      ``[0,1]``, in that order.  The boundary of the second is, on the
      one hand, computed the same way: the n-th face is obtained by
      omitting the n-th vertex.  On the other hand, the boundary is
      explicitly declared to be edges ``[0,1]``, ``[0,2]``, and
      ``[1,2]``, in that order.  This glues the second triangle to the
      first in the prescribed way.  The three edges each start and end
      at the single vertex, ``Simplex(0)``.

      .. image:: ../../media/torus_labelled.png

    - ``data`` may be nested lists or tuples.  The n-th entry in the
      list is a list of the n-simplices in the complex, and each
      n-simplex is encoded as a list, the i-th entry of which is its
      i-th face.  Each face is represented by an integer, giving its
      index in the list of (n-1)-faces.  For example, consider this::

        sage: P = DeltaComplex( [ [(), ()],  [(1,0), (1,0), (0,0)],
        ....:                     [(1,0,2), (0, 1, 2)] ])

      The 0th entry in the list is ``[(), ()]``: there are two
      0-simplices, and their boundaries are empty.

      The 1st entry in the list is ``[(1,0), (1,0), (0,0)]``: there
      are three 1-simplices.  Two of them have boundary ``(1,0)``,
      which means that their 0th face is vertex 1 (in the list of
      vertices), and their 1st face is vertex 0.  The other edge has
      boundary ``(0,0)``, so it starts and ends at vertex 0.

      The 2nd entry in the list is ``[(1,0,2), (0,1,2)]``: there are
      two 2-simplices.  The first 2-simplex has boundary ``(1,0,2)``,
      meaning that its 0th face is edge 1 (in the list above), its 1st
      face is edge 0, and its 2nd face is edge 2; similarly for the
      2nd 2-simplex.

      If one draws two triangles and identifies them according to this
      description, the result is the real projective plane.

      .. image:: ../../media/rp2.png

      ::

        sage: P.homology(1)                                                             # needs sage.modules
        C2
        sage: P.cohomology(2)                                                           # needs sage.modules
        C2

      Closely related to this form for ``data`` is ``X.cells()``
      for a `\Delta`-complex ``X``: this is a dictionary, indexed by
      dimension ``d``, whose ``d``-th entry is a list of the
      ``d``-simplices, as a list::

        sage: P.cells()
        {-1: ((),),
         0: ((), ()),
         1: ((1, 0), (1, 0), (0, 0)),
         2: ((1, 0, 2), (0, 1, 2))}

    - ``data`` may be a dictionary indexed by integers.  For each
      integer `n`, the entry with key `n` is the list of
      `n`-simplices: this is the same format as is output by the
      :meth:`cells` method. ::

        sage: P = DeltaComplex( [ [(), ()],  [(1,0), (1,0), (0,0)],
        ....:                     [(1,0,2), (0, 1, 2)] ])
        sage: cells_dict = P.cells()
        sage: cells_dict
        {-1: ((),),
         0: ((), ()),
         1: ((1, 0), (1, 0), (0, 0)),
         2: ((1, 0, 2), (0, 1, 2))}
        sage: DeltaComplex(cells_dict)
        Delta complex with 2 vertices and 8 simplices
        sage: P == DeltaComplex(cells_dict)
        True

    Since `\Delta`-complexes are generalizations of simplicial
    complexes, any simplicial complex may be viewed as a
    `\Delta`-complex::

        sage: RP2 = simplicial_complexes.RealProjectivePlane()
        sage: RP2_delta = RP2.delta_complex()
        sage: RP2.f_vector()
        [1, 6, 15, 10]
        sage: RP2_delta.f_vector()
        [1, 6, 15, 10]

    Finally, `\Delta`-complex constructions for several familiar
    spaces are available as follows::

        sage: delta_complexes.Sphere(4)  # the 4-sphere
        Delta complex with 5 vertices and 33 simplices
        sage: delta_complexes.KleinBottle()
        Delta complex with 1 vertex and 7 simplices
        sage: delta_complexes.RealProjectivePlane()
        Delta complex with 2 vertices and 8 simplices

    Type ``delta_complexes.`` and then hit the :kbd:`Tab` key to get the
    full list.
    """
    def __init__(self, data=None, check_validity=True):
        r"""
        Define a `\Delta`-complex.  See :class:`DeltaComplex` for more
        documentation.

        EXAMPLES::

            sage: X = DeltaComplex({Simplex(3):True, Simplex(range(1,5)): Simplex(3), Simplex(range(2,6)): Simplex(3)}); X  # indirect doctest
            Delta complex with 4 vertices and 18 simplices
            sage: X.homology()                                                          # needs sage.modules
            {0: 0, 1: 0, 2: 0, 3: Z x Z}
            sage: X == loads(dumps(X))
            True
        """
        def store_bdry(simplex, faces):
            r"""
            Given a simplex of dimension d and a list of boundaries
            (as other simplices), this stores each boundary face in
            new_data[d-1] if necessary, records the index of each
            boundary face in bdry_list, represents the simplex as
            bdry_list in new_data[d], and returns bdry_list.

            If the simplex is in the dictionary old_delayed, then it
            is already stored, temporarily, in new_data[d], so replace
            its temporary version with bdry_list.
            """
            bdry_list = []
            d = simplex.dimension()
            if d > 0:
                for f in faces:
                    if f in new_data[d-1]:
                        bdry_list.append(new_data[d-1].index(f))
                    else:
                        bdry_list.append(len(new_data[d-1]))
                        new_delayed[f] = len(new_data[d-1])
                        new_data[d-1].append(f)
                bdry_list = tuple(bdry_list)
            else:
                bdry_list = ()
            if simplex in old_delayed:
                idx = old_delayed[simplex]
                new_data[d][idx] = bdry_list
            else:
                new_data[d].append(bdry_list)
            return bdry_list

        new_data = {-1: ((),)}  # add the empty cell
        if data is None:
            pass
        else:
            if isinstance(data, (list, tuple)):
                dim = 0
                for s in data:
                    new_data[dim] = s
                    dim += 1
            elif isinstance(data, dict):
                if all(isinstance(a, (int, Integer)) for a in data):
                    # a dictionary indexed by integers
                    new_data = data
                    if -1 not in new_data:
                        new_data[-1] = ((),)  # add the empty cell
                else:
                    # else a dictionary indexed by simplices
                    dimension = max([f.dimension() for f in data])
                    old_data_by_dim = {}
                    for dim in range(dimension + 1):
                        old_data_by_dim[dim] = []
                        new_data[dim] = []
                    for x in data:
                        if not isinstance(x, Simplex):
                            raise TypeError("each key in the data dictionary must be a simplex")
                        old_data_by_dim[x.dimension()].append(x)
                    old_delayed = {}
                    for dim in range(dimension, -1, -1):
                        new_delayed = {}
                        current = {}
                        for x in old_data_by_dim[dim]:
                            if x in data:
                                bdry = data[x]
                            else:
                                bdry = True
                            if isinstance(bdry, Simplex):
                                # case 1
                                # value is a simplex, so x is glued to the old
                                # one along its boundary.  So the boundary of
                                # x is the boundary of the old simplex.
                                if bdry in current:
                                    # if the old simplex is there, copy its boundary
                                    if x in old_delayed:
                                        idx = old_delayed[x]
                                        new_data[dim][idx] = current[bdry]
                                    else:
                                        new_data[dim].append(current[bdry])
                                elif bdry in data:
                                    # the old simplex has not yet been added to
                                    # new_data, but is in the data dictionary.  So
                                    # add it.
                                    current[bdry] = store_bdry(bdry, bdry.faces())
                                    new_data[dim].append(current[bdry])
                                else:
                                    raise ValueError("in the data dictionary, there is a value which is a simplex not already in the dictionary")
                            elif isinstance(bdry, (list, tuple)):
                                # case 2
                                # boundary is a list or tuple
                                current[x] = store_bdry(x, bdry)
                            else:
                                # case 3
                                # no valid boundary specified, so the default
                                # boundary of x should be used
                                if x not in current:
                                    # x hasn't already been added, in case 1
                                    current[x] = store_bdry(x, x.faces())
                        old_delayed = new_delayed
                        if dim > 0:
                            old_data_by_dim[dim-1].extend(old_delayed.keys())
            else:
                raise ValueError("data is not a list, tuple, or dictionary")
        for n in new_data:
            new_data[n] = tuple(new_data[n])
        # at this point, new_data is a dictionary indexed by
        # dimension, with new_data[d] a list of "simplices" in
        # dimension d
        if check_validity:
            dim = max(new_data)
            for d in range(dim, 1, -1):
                for s in new_data[d]:  # s is a d-simplex
                    faces = new_data[d-1]
                    for j in range(d+1):
                        if not all(faces[s[j]][i] == faces[s[i]][j-1] for i in range(j)):
                            msg = "simplicial identity d_i d_j = d_{j-1} d_i fails"
                            msg += " for j={}, in dimension {}".format(j, d)
                            raise ValueError(msg)
        # self._cells_dict: dictionary indexed by dimension d: for
        # each d, have list or tuple of simplices, and for each
        # simplex, have list or tuple with its boundary (as the index
        # of an element in the list of (d-1)-simplices).
        self._cells_dict = new_data
        # self._is_subcomplex_of: if self is a subcomplex of another
        # Delta complex, record that other complex here, along with
        # data relating the cells in self to the cells in the
        # containing complex: for each dimension, a list of indices
        # specifying, for each cell in self, which cell it corresponds
        # to in the containing complex.
        self._is_subcomplex_of = None
        # self._complex: dictionary indexed by dimension d, base_ring,
        # etc.: differential from dim d to dim d-1 in the associated
        # chain complex.  thus to get the differential in the cochain
        # complex from dim d-1 to dim d, take the transpose of this
        # one.
        # self._complex = {}

    def subcomplex(self, data):
        r"""
        Create a subcomplex.

        INPUT:

        - ``data`` -- dictionary indexed by dimension or a list (or
          tuple); in either case, data[n] should be the list (or tuple
          or set) of the indices of the simplices to be included in
          the subcomplex

        This automatically includes all faces of the simplices in
        ``data``, so you only have to specify the simplices which are
        maximal with respect to inclusion.

        EXAMPLES::

            sage: X = delta_complexes.Torus()
            sage: A = X.subcomplex({2: [0]})  # one of the triangles of X
            sage: X.homology(subcomplex=A)                                              # needs sage.modules
            {0: 0, 1: 0, 2: Z}

        In the following, ``line`` is a line segment and ``ends`` is
        the complex consisting of its two endpoints, so the relative
        homology of the two is isomorphic to the homology of a circle::

            sage: line = delta_complexes.Simplex(1) # an edge
            sage: line.cells()
            {-1: ((),), 0: ((), ()), 1: ((0, 1),)}
            sage: ends = line.subcomplex({0: (0, 1)})
            sage: ends.cells()
            {-1: ((),), 0: ((), ())}
            sage: line.homology(subcomplex=ends)                                        # needs sage.modules
            {0: 0, 1: Z}
        """
        if isinstance(data, (list, tuple)):
            data = dict(zip(range(len(data)), data))

        # new_dict: dictionary for constructing the subcomplex
        new_dict = {}
        # new_data: dictionary of all cells in the subcomplex: store
        # this with the subcomplex to make it fast to list the cells
        # in self which are not in the subcomplex.
        new_data = {}
        # max_dim: maximum dimension of cells being added
        max_dim = max(data.keys())
        # cells_to_add: in each dimension, add these cells to
        # new_dict.  start with the cells given in new_data and add
        # faces of cells one dimension higher.
        cells_to_add = data[max_dim]
        cells = self.cells()
        for d in range(max_dim, -1, -1):
            # cells_to_add is the set of indices of d-cells in self to
            # add to new_dict.
            cells_to_add = sorted(cells_to_add)
            # we add only these cells, so we need to translate their
            # indices from, for example, (0, 1, 4, 5) to (0, 1, 2, 3).
            # That is, when they appear as boundaries of (d+1)-cells,
            # we need to translate their indices in each (d+1)-cell.
            # Here is the key for that translation:
            translate = dict(zip(cells_to_add, range(len(cells_to_add))))
            new_dict[d] = []
            d_cells = cells_to_add
            new_data[d] = cells_to_add
            try:
                cells_to_add = set(new_data[d-1])  # begin to populate the (d-1)-cells
            except KeyError:
                cells_to_add = set()
            for x in d_cells:
                if d+1 in new_dict:
                    old = new_dict[d+1]
                    new_dict[d+1] = []
                    for f in old:
                        new_dict[d+1].append(tuple([translate[n] for n in f]))
                new_dict[d].append(cells[d][x])
                cells_to_add.update(cells[d][x])
        new_cells = [new_dict[n] for n in range(max_dim + 1)]
        sub = DeltaComplex(new_cells)
        sub._is_subcomplex_of = {self: new_data}
        return sub

    def __hash__(self):
        r"""
        TESTS::

            sage: hash(delta_complexes.Sphere(2)) == hash(delta_complexes.Sphere(2))
            True
            sage: hash(delta_complexes.Sphere(4)) == hash(delta_complexes.Sphere(4))
            True
        """
        return hash(frozenset(self._cells_dict.items()))

    def __eq__(self, right):
        r"""
        Two `\Delta`-complexes are equal, according to this, if they have
        the same ``_cells_dict``.

        EXAMPLES::

            sage: S4 = delta_complexes.Sphere(4)
            sage: S2 = delta_complexes.Sphere(2)
            sage: S4 == S2
            False
            sage: newS2 = DeltaComplex({Simplex(2):True, Simplex([8,12,17]): Simplex(2)})
            sage: newS2 == S2
            True
        """
        return self._cells_dict == right._cells_dict

    def __ne__(self, other):
        r"""
        Return ``True`` if ``self`` and ``other`` are not equal.

        EXAMPLES::

            sage: S4 = delta_complexes.Sphere(4)
            sage: S2 = delta_complexes.Sphere(2)
            sage: S4 != S2
            True
            sage: newS2 = DeltaComplex({Simplex(2):True, Simplex([8,12,17]): Simplex(2)})
            sage: newS2 != S2
            False
        """
        return not self.__eq__(other)

    def cells(self, subcomplex=None):
        r"""
        The cells of this `\Delta`-complex.

        INPUT:

        - ``subcomplex`` -- a subcomplex of this complex (default: ``None``)

        The cells of this `\Delta`-complex, in the form of a dictionary:
        the keys are integers, representing dimension, and the value
        associated to an integer d is the list of d-cells.  Each
        d-cell is further represented by a list, the i-th entry of
        which gives the index of its i-th face in the list of
        (d-1)-cells.

        If the optional argument ``subcomplex`` is present, then
        "return only the faces which are *not* in the subcomplex".  To
        preserve the indexing, which is necessary to compute the
        relative chain complex, this actually replaces the faces in
        ``subcomplex`` with ``None``.

        EXAMPLES::

            sage: S2 = delta_complexes.Sphere(2)
            sage: S2.cells()
            {-1: ((),),
             0: ((), (), ()),
             1: ((0, 1), (0, 2), (1, 2)),
             2: ((0, 1, 2), (0, 1, 2))}
            sage: A = S2.subcomplex({1: [0,2]}) # one edge
            sage: S2.cells(subcomplex=A)
            {-1: (None,),
             0: (None, None, None),
             1: (None, (0, 2), None),
             2: ((0, 1, 2), (0, 1, 2))}
        """
        cells = self._cells_dict.copy()
        if subcomplex is None:
            return cells
        if subcomplex._is_subcomplex_of is None or self not in subcomplex._is_subcomplex_of:
            if subcomplex == self:
                for d in range(-1, max(cells.keys())+1):
                    l = len(cells[d])
                    cells[d] = [None]*l   # get rid of all cells
                return cells
            else:
                raise ValueError("this is not a subcomplex of self")
        else:
            subcomplex_cells = subcomplex._is_subcomplex_of[self]
            for d in range(max(subcomplex_cells.keys()) + 1):
                L = list(cells[d])
                for c in subcomplex_cells[d]:
                    L[c] = None
                cells[d] = tuple(L)
            cells[-1] = (None,)
        return cells

    def chain_complex(self, subcomplex=None, augmented=False,
                      verbose=False, check=False, dimensions=None,
                      base_ring=ZZ, cochain=False):
        r"""
        The chain complex associated to this `\Delta`-complex.

        INPUT:

        - ``dimensions`` -- if ``None``, compute the chain complex in all
          dimensions.  If a list or tuple of integers, compute the
          chain complex in those dimensions, setting the chain groups
          in all other dimensions to zero.  NOT IMPLEMENTED YET: this
          function always returns the entire chain complex
        - ``base_ring`` -- commutative ring (default: ``ZZ``)
        - ``subcomplex`` -- a subcomplex of this simplicial complex (default:
          empty). Compute the chain complex relative to this subcomplex.
        - ``augmented`` -- boolean (default: ``False``); if ``True``, return the
          augmented chain complex (that is, include a class in dimension `-1`
          corresponding to the empty cell).  This is ignored if ``dimensions``
          is specified or if ``subcomplex`` is nonempty.
        - ``cochain`` -- boolean (default: ``False``); if ``True``, return the
          cochain complex (that is, the dual of the chain complex)
        - ``verbose`` -- boolean (default: ``False``); if ``True``, print some
          messages as the chain complex is computed
        - ``check`` -- boolean (default: ``False``); if ``True``, make sure that
          the chain complex is actually a chain complex: the differentials are
          composable and their product is zero

        .. NOTE::

           If subcomplex is nonempty, then the argument ``augmented``
           has no effect: the chain complex relative to a nonempty
           subcomplex is zero in dimension `-1`.

        EXAMPLES::

            sage: # needs sage.modules
            sage: circle = delta_complexes.Sphere(1)
            sage: circle.chain_complex()
            Chain complex with at most 2 nonzero terms over Integer Ring
            sage: circle.chain_complex()._latex_()
            '\\Bold{Z}^{1} \\xrightarrow{d_{1}} \\Bold{Z}^{1}'
            sage: circle.chain_complex(base_ring=QQ, augmented=True)
            Chain complex with at most 3 nonzero terms over Rational Field
            sage: circle.homology(dim=1)
            Z
            sage: circle.cohomology(dim=1)
            Z
            sage: T = delta_complexes.Torus()
            sage: T.chain_complex(subcomplex=T)
            Trivial chain complex over Integer Ring
            sage: T.homology(subcomplex=T)
            {0: 0, 1: 0, 2: 0}
            sage: A = T.subcomplex({2: [1]})  # one of the two triangles forming T
            sage: T.chain_complex(subcomplex=A)
            Chain complex with at most 1 nonzero terms over Integer Ring
            sage: T.homology(subcomplex=A)
            {0: 0, 1: 0, 2: Z}
        """
        from sage.homology.chain_complex import ChainComplex

        if subcomplex is not None:
            # relative chain complex, so don't augment the chain complex
            augmented = False

        differentials = {}
        if augmented:
            empty_simplex = 1  # number of (-1)-dimensional simplices
        else:
            empty_simplex = 0
        vertices = self.n_cells(0, subcomplex=subcomplex)
        old = vertices
        old_real = [x for x in old if x is not None]  # remove faces not in subcomplex
        n = len(old_real)
        differentials[0] = matrix(base_ring, empty_simplex, n, n*empty_simplex*[1])
        # current is list of simplices in dimension dim
        # current_real is list of simplices in dimension dim, with None filtered out
        # old is list of simplices in dimension dim-1
        # old_real is list of simplices in dimension dim-1, with None filtered out
        for dim in range(1, self.dimension()+1):
            current = list(self.n_cells(dim, subcomplex=subcomplex))
            current_real = [x for x in current if x is not None]
            i = 0
            i_real = 0
            translate = {}
            for s in old:
                if s is not None:
                    translate[i] = i_real
                    i_real += 1
                i += 1
            mat_dict = {}
            col = 0
            for s in current_real:
                sign = 1
                for row in s:
                    if old[row] is not None:
                        actual_row = translate[row]
                        if (actual_row, col) in mat_dict:
                            mat_dict[(actual_row, col)] += sign
                        else:
                            mat_dict[(actual_row, col)] = sign
                    sign *= -1
                col += 1
            differentials[dim] = matrix(base_ring, len(old_real), len(current_real), mat_dict)
            old = current
            old_real = current_real
        if cochain:
            cochain_diffs = {}
            for dim in differentials:
                cochain_diffs[dim-1] = differentials[dim].transpose()
            return ChainComplex(data=cochain_diffs, degree=1,
                                base_ring=base_ring, check=check)
        else:
            return ChainComplex(data=differentials, degree=-1,
                                base_ring=base_ring, check=check)

    def alexander_whitney(self, cell, dim_left):
        r"""
        Subdivide ``cell`` in this `\Delta`-complex into a pair of
        simplices.

        For an abstract simplex with vertices `v_0`, `v_1`, ...,
        `v_n`, then subdivide it into simplices `(v_0, v_1, ...,
        v_{dim_left})` and `(v_{dim_left}, v_{dim_left + 1}, ...,
        v_n)`. In a `\Delta`-complex, instead take iterated faces:
        take top faces to get the left factor, take bottom faces to
        get the right factor.

        INPUT:

        - ``cell`` -- a simplex in this complex, given as a pair
          ``(idx, tuple)``, where ``idx`` is its index in the list of
          cells in the given dimension, and ``tuple`` is the tuple of
          its faces

        - ``dim_left`` -- integer between 0 and one more than the
          dimension of this simplex

        OUTPUT: list containing just the triple ``(1, left,
        right)``, where ``left`` and ``right`` are the two cells
        described above, each given as pairs ``(idx, tuple)``.

        EXAMPLES::

            sage: X = delta_complexes.Torus()
            sage: X.n_cells(2)
            [(1, 2, 0), (0, 2, 1)]
            sage: X.alexander_whitney((0, (1, 2, 0)), 1)
            [(1, (0, (0, 0)), (1, (0, 0)))]
            sage: X.alexander_whitney((0, (1, 2, 0)), 0)
            [(1, (0, ()), (0, (1, 2, 0)))]
            sage: X.alexander_whitney((1, (0, 2, 1)), 2)
            [(1, (1, (0, 2, 1)), (0, ()))]
        """
        dim = len(cell[1]) - 1
        left_cell = cell[1]
        idx_l = cell[0]
        for i in range(dim, dim_left, -1):
            idx_l = left_cell[i]
            left_cell = self.n_cells(i-1)[idx_l]
        right_cell = cell[1]
        idx_r = cell[0]
        for i in range(dim, dim - dim_left, -1):
            idx_r = right_cell[0]
            right_cell = self.n_cells(i-1)[idx_r]
        return [(ZZ.one(), (idx_l, left_cell), (idx_r, right_cell))]

    def n_skeleton(self, n):
        r"""
        The n-skeleton of this `\Delta`-complex.

        - ``n`` -- nonnegative integer; dimension

        EXAMPLES::

            sage: S3 = delta_complexes.Sphere(3)
            sage: S3.n_skeleton(1) # 1-skeleton of a tetrahedron
            Delta complex with 4 vertices and 11 simplices
            sage: S3.n_skeleton(1).dimension()
            1
            sage: S3.n_skeleton(1).homology()                                           # needs sage.modules
            {0: 0, 1: Z x Z x Z}
        """
        if n >= self.dimension():
            return self
        data = [self._cells_dict[d] for d in range(n + 1)]
        return DeltaComplex(data)

    def graph(self):
        r"""
        The 1-skeleton of this `\Delta`-complex as a graph.

        EXAMPLES::

            sage: T = delta_complexes.Torus()
            sage: T.graph()
            Looped multi-graph on 1 vertex
            sage: S = delta_complexes.Sphere(2)
            sage: S.graph()
            Graph on 3 vertices
            sage: delta_complexes.Simplex(4).graph() == graphs.CompleteGraph(5)
            True
        """
        from sage.graphs.graph import Graph

        data = {}
        for vertex in range(len(self.n_cells(0))):
            data[vertex] = []
        for edge in self.n_cells(1):
            data[edge[0]].append(edge[1])
        return Graph(data)

    def join(self, other):
        r"""
        The join of this `\Delta`-complex with another one.

        INPUT:

        - ``other`` -- another `\Delta`-complex (the right-hand factor)

        OUTPUT: the join ``self * other``

        The join of two `\Delta`-complexes `S` and `T` is the
        `\Delta`-complex `S*T` with simplices of the form `[v_0, ...,
        v_k, w_0, ..., w_n]` for all simplices `[v_0, ..., v_k]` in
        `S` and `[w_0, ..., w_n]` in `T`.  The faces are computed
        accordingly: the i-th face of such a simplex is either `(d_i S)
        * T` if `i \leq k`, or `S * (d_{i-k-1} T)` if `i > k`.

        EXAMPLES::

            sage: T = delta_complexes.Torus()
            sage: S0 = delta_complexes.Sphere(0)
            sage: T.join(S0)  # the suspension of T
            Delta complex with 3 vertices and 21 simplices

        Compare to simplicial complexes::

            sage: K = delta_complexes.KleinBottle()
            sage: T_simp = simplicial_complexes.Torus()
            sage: K_simp = simplicial_complexes.KleinBottle()
            sage: T.join(K).homology()[3] == T_simp.join(K_simp).homology()[3]  # long time (3 seconds), needs sage.modules
            True

        The notation '*' may be used, as well::

            sage: S1 = delta_complexes.Sphere(1)
            sage: X = S1 * S1    # X is a 3-sphere
            sage: X.homology()                                                          # needs sage.modules
            {0: 0, 1: 0, 2: 0, 3: Z}
        """
        data = []
        # vertices of the join: the union of the vertices.  put the
        # vertices of self first, then the vertices of right.
        data.append(self.n_cells(0) + other.n_cells(0))
        bdries = {}
        for l_idx in range(len(self.n_cells(0))):
            bdries[(0, l_idx, -1, 0)] = l_idx
        for r_idx in range(len(other.n_cells(0))):
            bdries[(-1, 0, 0, r_idx)] = len(self.n_cells(0)) + r_idx
        # dimension of the join:
        maxdim = self.dimension() + other.dimension() + 1
        # now for the d-cells, d>0:
        for d in range(1, maxdim+1):
            d_cells = []
            positions = {}
            new_idx = 0
            for k in range(-1, d+1):
                n = d-1-k
                # d=n+k.  need a k-cell from self and an n-cell from other
                if k == -1:
                    left = [()]
                else:
                    left = self.n_cells(k)
                l_idx = 0
                if n == -1:
                    right = [()]
                else:
                    right = other.n_cells(n)
                for l in left:
                    r_idx = 0
                    for r in right:
                        # store index of the new simplex in positions
                        positions[(k, l_idx, n, r_idx)] = new_idx
                        # form boundary of l*r and store it in d_cells
                        bdry = []
                        # first faces come from left-hand factor
                        if k == 0:
                            bdry.append(bdries[(-1, 0, n, r_idx)])
                        else:
                            for i in range(k+1):
                                bdry.append(bdries[(k-1, l[i], n, r_idx)])
                        # remaining faces come from right-hand factor
                        if n == 0:
                            bdry.append(bdries[(k, l_idx, -1, 0)])
                        else:
                            for i in range(n+1):
                                bdry.append(bdries[(k, l_idx, n-1, r[i])])
                        d_cells.append(tuple(bdry))
                        r_idx += 1
                        new_idx += 1
                    l_idx += 1
            data.append(d_cells)
            bdries = positions
        return DeltaComplex(data)

    # Use * to mean 'join':
    __mul__ = join

    def cone(self):
        r"""
        The cone on this `\Delta`-complex.

        The cone is the complex formed by adding a new vertex `C` and
        simplices of the form `[C, v_0, ..., v_k]` for every simplex
        `[v_0, ..., v_k]` in the original complex.  That is, the cone
        is the join of the original complex with a one-point complex.

        EXAMPLES::

            sage: K = delta_complexes.KleinBottle()
            sage: K.cone()
            Delta complex with 2 vertices and 14 simplices
            sage: K.cone().homology()                                                   # needs sage.modules
            {0: 0, 1: 0, 2: 0, 3: 0}
        """
        return self.join(delta_complexes.Simplex(0))

    def suspension(self, n=1):
        r"""
        The suspension of this `\Delta`-complex.

        - ``n`` -- positive integer (default: 1); suspend this many times

        The suspension is the complex formed by adding two new
        vertices `S_0` and `S_1` and simplices of the form `[S_0, v_0,
        ..., v_k]` and `[S_1, v_0, ..., v_k]` for every simplex `[v_0,
        ..., v_k]` in the original complex.  That is, the suspension
        is the join of the original complex with a two-point complex
        (the 0-sphere).

        EXAMPLES::

            sage: S = delta_complexes.Sphere(0)
            sage: S3 = S.suspension(3)  # the 3-sphere
            sage: S3.homology()                                                         # needs sage.modules
            {0: 0, 1: 0, 2: 0, 3: Z}
        """
        if n < 0:
            raise ValueError("n must be nonnegative")
        if n == 0:
            return self
        if n == 1:
            return self.join(delta_complexes.Sphere(0))
        return self.suspension().suspension(int(n-1))

    def product(self, other):
        r"""
        The product of this `\Delta`-complex with another one.

        INPUT:

        - ``other`` -- another `\Delta`-complex (the right-hand factor)

        OUTPUT: the product ``self x other``

        .. WARNING::

           If ``X`` and ``Y`` are `\Delta`-complexes, then ``X*Y``
           returns their join, not their product.

        EXAMPLES::

            sage: K = delta_complexes.KleinBottle()
            sage: X = K.product(K)

            sage: # needs sage.modules
            sage: X.homology(1)
            Z x Z x C2 x C2
            sage: X.homology(2)
            Z x C2 x C2 x C2
            sage: X.homology(3)
            C2
            sage: X.homology(4)
            0
            sage: X.homology(base_ring=GF(2))
            {0: Vector space of dimension 0 over Finite Field of size 2,
             1: Vector space of dimension 4 over Finite Field of size 2,
             2: Vector space of dimension 6 over Finite Field of size 2,
             3: Vector space of dimension 4 over Finite Field of size 2,
             4: Vector space of dimension 1 over Finite Field of size 2}

            sage: S1 = delta_complexes.Sphere(1)
            sage: K.product(S1).homology() == S1.product(K).homology()                  # needs sage.modules
            True
            sage: S1.product(S1) == delta_complexes.Torus()
            True
        """
        data = []
        bdries = {}
        # vertices: the vertices in the product are of the form (v,w)
        # for v a vertex in self, w a vertex in other
        vertices = []
        l_idx = 0
        for v in self.n_cells(0):
            r_idx = 0
            for w in other.n_cells(0):
                # one vertex for each pair (v,w)
                # store its indices in bdries; store its boundary in vertices
                bdries[(0, l_idx, 0, r_idx, ((0, 0),))] = len(vertices)
                vertices.append(())  # add new vertex (simplex with empty bdry)
                r_idx += 1
            l_idx += 1
        data.append(tuple(vertices))
        # dim of the product:
        maxdim = self.dimension() + other.dimension()
        # d-cells, d>0: these are obtained by taking products of cells
        # of dimensions k and n, where n+k >= d and n <= d, k <= d.
        simplices = []
        new = {}
        for d in range(1, maxdim+1):
            for k in range(d+1):
                for n in range(d-k, d+1):
                    k_idx = 0
                    for k_cell in self.n_cells(k):
                        n_idx = 0
                        for n_cell in other.n_cells(n):
                            # find d-dimensional faces in product of
                            # k_cell and n_cell.  to avoid repetition,
                            # only look for faces which use all
                            # vertices of each factor: the 'path'
                            # corresponding to each d-cell must hit
                            # every row and every column in the
                            # lattice.  (See the 'product' method for
                            # Simplex, as well as the function
                            # 'lattice_paths', in
                            # simplicial_complex.py.)
                            for path in lattice_paths(list(range(k + 1)),
                                                      list(range(n + 1)),
                                                      length=d+1):
                                path = tuple(path)
                                new[(k, k_idx, n, n_idx, path)] = len(simplices)
                                bdry_list = []
                                for i in range(d+1):
                                    face_path = path[:i] + path[i+1:]
                                    if ((i < d and path[i][0] == path[i+1][0]) or
                                            (i > 0 and path[i][0] == path[i-1][0])):
                                        # this k-simplex
                                        k_face_idx = k_idx
                                        k_face_dim = k
                                    else:
                                        # face of this k-simplex
                                        k_face_idx = k_cell[path[i][0]]
                                        k_face_dim = k-1
                                        tail = []
                                        for j in range(i, d):
                                            tail.append((face_path[j][0]-1,
                                                         face_path[j][1]))
                                        face_path = face_path[:i] + tuple(tail)
                                    if ((i < d and path[i][1] == path[i+1][1]) or
                                            (i > 0 and path[i][1] == path[i-1][1])):
                                        # this n-simplex
                                        n_face_idx = n_idx
                                        n_face_dim = n
                                    else:
                                        # face of this n-simplex
                                        n_face_idx = n_cell[path[i][1]]
                                        n_face_dim = n-1
                                        tail = []
                                        for j in range(i, d):
                                            tail.append((face_path[j][0],
                                                         face_path[j][1]-1))
                                        face_path = face_path[:i] + tuple(tail)
                                    bdry_list.append(bdries[(k_face_dim, k_face_idx,
                                                             n_face_dim, n_face_idx,
                                                             face_path)])
                                simplices.append(tuple(bdry_list))
                            n_idx += 1
                        k_idx += 1
            # add d-simplices to data, store d-simplices in bdries,
            # reset simplices
            data.append(tuple(simplices))
            bdries = new
            new = {}
            simplices = []
        return DeltaComplex(data)

    def disjoint_union(self, right):
        r"""
        The disjoint union of this `\Delta`-complex with another one.

        INPUT:

        - ``right`` -- the other `\Delta`-complex (the right-hand factor)

        EXAMPLES::

            sage: S1 = delta_complexes.Sphere(1)
            sage: S2 = delta_complexes.Sphere(2)
            sage: S1.disjoint_union(S2).homology()                                      # needs sage.modules
            {0: Z, 1: Z, 2: Z}
        """
        dim = max(self.dimension(), right.dimension())
        data = {}
        # in dimension n, append simplices of self with simplices of
        # right, but translate each entry of each right simplex: add
        # len(self.n_cells(n-1)) to it
        for n in range(dim, 0, -1):
            data[n] = list(self.n_cells(n))
            translate = len(self.n_cells(n-1))
            for f in right.n_cells(n):
                data[n].append(tuple([a+translate for a in f]))
        data[0] = self.n_cells(0) + right.n_cells(0)
        return DeltaComplex(data)

    def wedge(self, right):
        r"""
        The wedge (one-point union) of this `\Delta`-complex with
        another one.

        - ``right`` -- the other `\Delta`-complex (the right-hand factor)

        .. NOTE::

            This operation is not well-defined if ``self`` or
            ``other`` is not path-connected.

        EXAMPLES::

            sage: S1 = delta_complexes.Sphere(1)
            sage: S2 = delta_complexes.Sphere(2)
            sage: S1.wedge(S2).homology()                                               # needs sage.modules
            {0: 0, 1: Z, 2: Z}
        """
        data = self.disjoint_union(right).cells()
        left_verts = len(self.n_cells(0))
        translate = {i: i for i in range(left_verts)}
        translate[left_verts] = 0
        for i in range(left_verts + 1, left_verts + len(right.n_cells(0))):
            translate[i] = i - 1
        data[0] = data[0][:-1]
        edges = [[translate[a] for a in e] for e in data[1]]
        data[1] = edges
        return DeltaComplex(data)

    def connected_sum(self, other):
        r"""
        Return the connected sum of ``self`` with ``other``.

        INPUT:

        - ``other`` -- another `\Delta`-complex

        OUTPUT: the connected sum ``self # other``

        .. warning::

           This does not check that ``self`` and ``other`` are manifolds.
           It doesn't even check that their facets all have the same
           dimension.  It just chooses top-dimensional simplices from
           each complex, checks that they have the same dimension,
           removes them, and glues the remaining pieces together.
           Since a (more or less) random facet is chosen from each
           complex, this method may return random results if applied
           to non-manifolds, depending on which facet is chosen.

        ALGORITHM:

        Pick a top-dimensional simplex from each complex.  Check to
        see if there are any identifications on either simplex, using
        the :meth:`_is_glued` method.  If there are no
        identifications, remove the simplices and glue the remaining
        parts of complexes along their boundary.  If there are
        identifications on a simplex, subdivide it repeatedly (using
        :meth:`elementary_subdivision`) until some piece has no
        identifications.

        EXAMPLES::

            sage: # needs sage.modules
            sage: T = delta_complexes.Torus()
            sage: S2 = delta_complexes.Sphere(2)
            sage: T.connected_sum(S2).cohomology() == T.cohomology()
            True
            sage: RP2 = delta_complexes.RealProjectivePlane()
            sage: T.connected_sum(RP2).homology(1)
            Z x Z x C2
            sage: T.connected_sum(RP2).homology(2)
            0
            sage: RP2.connected_sum(RP2).connected_sum(RP2).homology(1)
            Z x Z x C2
        """
        if not self.dimension() == other.dimension():
            raise ValueError("complexes are not of the same dimension")
        dim = self.dimension()
        # Look at the last simplex in the list of top-dimensional
        # simplices for each complex.  If there are identifications on
        # either of these simplices, subdivide until there are no more
        # identifications.
        Left = self
        while Left._is_glued():
            Left = Left.elementary_subdivision()
        Right = other
        while Right._is_glued():
            Right = Right.elementary_subdivision()
        # remove last top-dimensional face from each one and glue.
        data = {}
        for n in Left.cells():
            data[n] = list(Left.cells()[n])
        right_cells = Right.cells()
        data[dim] = data[dim][:-1]
        left_simplex = Left.n_cells(dim)[-1]
        right_simplex = Right.n_cells(dim)[-1]
        # renaming: dictionary for translating all simplices of Right
        renaming = dict(zip(right_simplex, left_simplex))
        # process_now: cells to be reindexed and added to data
        process_now = right_cells[dim][:-1]
        for n in range(dim, 0, -1):
            # glued: dictionary of just the simplices being glued
            glued = copy(renaming)
            # process_later: cells one dim lower to be added to data
            process_later = []
            old_idx = 0
            new_idx = len(data[n-1])
            # build 'renaming'
            for s in right_cells[n-1]:
                if old_idx not in renaming:
                    process_later.append(s)
                    renaming[old_idx] = new_idx
                    new_idx += 1
                old_idx += 1
            # reindex all simplices to be processed and add them to data
            for s in process_now:
                data[n].append(tuple([renaming[i] for i in s]))
            # set up for next loop, one dimension down
            renaming = {}
            process_now = process_later
            for f in glued:
                renaming.update(dict(zip(right_cells[n-1][f], data[n-1][glued[f]])))
        # deal with vertices separately.  we just need to add enough
        # vertices: all the vertices from Right, minus the number
        # being glued, which should be dim+1, the number of vertices
        # in the simplex of dimension dim being glued.
        for i in range(len(right_cells[0]) - dim - 1):
            data[0].append(())
        return DeltaComplex(data)

    def elementary_subdivision(self, idx=-1):
        r"""
        Perform an "elementary subdivision" on a top-dimensional
        simplex in this `\Delta`-complex.  If the optional argument
        ``idx`` is present, it specifies the index (in the list of
        top-dimensional simplices) of the simplex to subdivide.  If
        not present, subdivide the last entry in this list.

        INPUT:

        - ``idx`` -- integer (default: -1); index specifying which simplex to
          subdivide

        OUTPUT: `\Delta`-complex with one simplex subdivided

        *Elementary subdivision* of a simplex means replacing that
        simplex with the cone on its boundary.  That is, given a
        `\Delta`-complex containing a `d`-simplex `S` with vertices
        `v_0`, ..., `v_d`, form a new `\Delta`-complex by

        - removing `S`
        - adding a vertex `w` (thought of as being in the interior of `S`)
        - adding all simplices with vertices `v_{i_0}`, ...,
          `v_{i_k}`, `w`, preserving any identifications present
          along the boundary of `S`

        The algorithm for achieving this uses
        :meth:`_epi_from_standard_simplex` to keep track of simplices
        (with multiplicity) and what their faces are: this method
        defines a surjection `\pi` from the standard `d`-simplex to
        `S`.  So first remove `S` and add a new vertex `w`, say at the
        end of the old list of vertices.  Then for each vertex `v` in
        the standard `d`-simplex, add an edge from `\pi(v)` to `w`;
        for each edge `(v_0, v_1)` in the standard `d`-simplex, add a
        triangle `(\pi(v_0), \pi(v_1), w)`, etc.

        Note that given an `n`-simplex `(v_0, v_1, ..., v_n)` in the
        standard `d`-simplex, the faces of the new `(n+1)`-simplex are
        given by removing vertices, one at a time, from `(\pi(v_0),
        ..., \pi(v_n), w)`.  These are either the image of the old
        `n`-simplex (if `w` is removed) or the various new
        `n`-simplices added in the previous dimension.  So keep track
        of what's added in dimension `n` for use in computing the
        faces in dimension `n+1`.

        In contrast with barycentric subdivision, note that only the
        interior of `S` has been changed; this allows for subdivision
        of a single top-dimensional simplex without subdividing every
        simplex in the complex.

        The term "elementary subdivision" is taken from p. 112 in John
        M. Lee's book [Lee2011]_.

        EXAMPLES::

            sage: T = delta_complexes.Torus()
            sage: T.n_cells(2)
            [(1, 2, 0), (0, 2, 1)]
            sage: T.elementary_subdivision(0)  # subdivide first triangle
            Delta complex with 2 vertices and 13 simplices
            sage: X = T.elementary_subdivision(); X  # subdivide last triangle
            Delta complex with 2 vertices and 13 simplices
            sage: X.elementary_subdivision()
            Delta complex with 3 vertices and 19 simplices
            sage: X.homology() == T.homology()                                          # needs sage.modules
            True
        """
        pi = self._epi_from_standard_simplex(idx=idx)
        cells_dict = {}
        old_cells = self.cells()
        for n in old_cells:
            cells_dict[n] = list(old_cells[n])
        dim = self.dimension()
        # cells of standard simplex of dimension dim
        std_cells = SimplicialComplex([Simplex(dim)]).delta_complex(sort_simplices=True).cells()
        # adjust zero-cells so they're distinct
        std_cells[0] = tuple([[n] for n in range(dim + 1)])
        # remove the cell being subdivided
        cells_dict[dim].pop(idx)
        # add the new vertex "w"
        cells_dict[0].append(())
        # added_cells: dict indexed by (n-1)-cells, with value the
        # corresponding new n-cell.
        added_cells = {(): len(cells_dict[0])-1}
        for n in range(dim):
            new_cells = {}
            # for each n-cell in the standard simplex, add an
            # (n+1)-cell to the subdivided complex.
            try:
                simplices = sorted(pi[n])
            except TypeError:
                simplices = pi[n]
            for simplex in simplices:
                # compute the faces of the new (n+1)-cell.
                cell = []
                for i in simplex:
                    if n > 0:
                        bdry = tuple(std_cells[n-1][i])
                    else:
                        bdry = ()
                    cell.append(added_cells[bdry])
                # last face is the image of the old simplex)
                cell.append(pi[n][simplex])
                cell = tuple(cell)
                cells_dict[n+1].append(cell)
                new_cells[simplex] = len(cells_dict[n+1])-1
            added_cells = new_cells
        return DeltaComplex(cells_dict)

    def _epi_from_standard_simplex(self, idx=-1, dim=None):
        r"""
        Construct an epimorphism from a standard simplex to a
        top-dimensional simplex in this `\Delta`-complex.

        If the optional argument ``dim`` is not ``None``, then
        construct the map to a simplex with this dimension.  If the
        optional argument ``idx`` is present, it specifies which
        simplex to use by giving its index in the list of simplices of
        the appropriate dimension; if not present, use the last
        simplex in this list.

        This is used by :meth:`elementary_subdivision`.

        INPUT:

        - ``idx`` -- integer (default: -1); index specifying which simplex to
          examine
        - ``dim`` -- integer (default: dimension of complex); dimension of simplex
          to consider

        OUTPUT: boolean; whether the boundary of the simplex has any
        identifications

        Suppose that the dimension is `d`. The map is given by a
        dictionary indexed by dimension: in dimension `i`, its value
        is a dictionary specifying, for each `i`-simplex in the
        domain, the corresponding `i`-simplex in the codomain.  The
        vertices are specified as their indices in the lists of
        simplices in each complex; the same goes for all of the
        simplices in the codomain.  The simplices of dimension 1 or
        higher in the domain are listed explicitly (in the form of
        entries from the output of :meth:`cells`).

        In this function, the "standard simplex" is defined to be
        ``simplicial_complexes.Simplex(d).delta_complex(sort_simplices=True)``.

        EXAMPLES:

        The `\Delta`-complex model for a torus has two triangles and
        three edges, but only one vertex.  So a surjection from the
        standard 2-simplex to either of the triangles is a bijection
        in dimension 1, but in dimension 0, sends all three vertices
        to the same place::

            sage: T = delta_complexes.Torus()
            sage: sorted(T._epi_from_standard_simplex()[1].items())
            [((1, 0), 1), ((2, 0), 2), ((2, 1), 0)]
            sage: sorted(T._epi_from_standard_simplex()[0].items())
            [((0,), 0), ((1,), 0), ((2,), 0)]
        """
        if dim is None:
            dim = self.dimension()
        # the output is easier to read if the entries are nonnegative.
        if idx == -1:
            idx = len(self.n_cells(dim)) - 1
        simplex = SimplicialComplex([Simplex(dim)]).delta_complex(sort_simplices=True)
        simplex_cells = simplex.cells()
        self_cells = self.cells()
        if dim > 0:
            mapping = {dim: {tuple(simplex_cells[dim][0]): idx}}
        else:
            mapping = {dim: {(0,): idx}}
        faces_dict = mapping[dim]
        for n in range(dim, 0, -1):
            n_cells = faces_dict
            faces_dict = {}
            for cell in n_cells:
                if n > 1:
                    faces = [tuple(simplex_cells[n-1][cell[j]]) for j in range(n+1)]
                    one_cell = dict(zip(faces, self_cells[n][n_cells[cell]]))
                else:
                    temp = dict(zip(cell, self_cells[n][n_cells[cell]]))
                    one_cell = {}
                    for j in temp:
                        one_cell[(j,)] = temp[j]
                for j in one_cell:
                    if j not in faces_dict:
                        faces_dict[j] = one_cell[j]
            mapping[n-1] = faces_dict
        return mapping

    def _is_glued(self, idx=-1, dim=None):
        r"""
        Return ``True`` if there is any gluing along the boundary of a
        top-dimensional simplex in this `\Delta`-complex.

        If the optional argument ``idx`` is present, it specifies
        which simplex to consider by giving its index in the list of
        top-dimensional simplices; if not present, look at the last
        simplex in this list.  If the optional argument ``dim`` is
        present, it specifies the dimension of the simplex to
        consider; if not present, look at a top-dimensional simplex.

        This is used by :meth:`connected_sum`.

        INPUT:

        - ``idx`` -- integer (default: -1); index specifying which simplex to
          examine
        - ``dim`` -- integer (default: dimension of complex); dimension of simplex
          to consider

        OUTPUT: boolean; whether the boundary of the simplex has any
        identifications

        EXAMPLES::

            sage: T = delta_complexes.Torus()
            sage: T._is_glued()
            True
            sage: S = delta_complexes.Simplex(3)
            sage: S._is_glued()
            False
        """
        if dim is None:
            dim = self.dimension()

        simplex = self.n_cells(dim)[idx]
        i = self.dimension() - 1
        i_faces = set(simplex)
        # if there are enough i_faces, then no gluing is evident so far
        not_glued = (len(i_faces) == binomial(dim+1, i+1))
        while not_glued and i > 0:
            # count the (i-1) cells and compare to (n+1) choose i.
            old_faces = i_faces
            i_faces = set()
            all_cells = self.n_cells(i)
            for face in old_faces:
                i_faces.update(all_cells[face])
            not_glued = (len(i_faces) == binomial(dim+1, i))
            i = i-1
        return not not_glued

    def face_poset(self):
        r"""
        The face poset of this `\Delta`-complex, the poset of
        nonempty cells, ordered by inclusion.

        EXAMPLES::

            sage: T = delta_complexes.Torus()
            sage: T.face_poset()
            Finite poset containing 6 elements
        """
        from sage.combinat.posets.posets import Poset
        # given the structure of self.cells(), it's easier to compute
        # the dual poset, then reverse it at the end.
        dim = self.dimension()
        covers = {}
        # store each n-simplex as a pair (n, idx).
        for n in range(dim, 0, -1):
            idx = 0
            for s in self.n_cells(n):
                covers[(n, idx)] = list({(n-1, i) for i in s})
                idx += 1
        # deal with vertices separately: they have no covers (in the
        # dual poset).
        idx = 0
        for s in self.n_cells(0):
            covers[(0, idx)] = []
            idx += 1
        return Poset(Poset(covers).hasse_diagram().reverse())

    # implement using the definition?  the simplices are obtained by
    # taking chains of inclusions of simplices, etc.  have to work out
    # the faces and identifications.
    def barycentric_subdivision(self):
        r"""
        Not implemented.

        EXAMPLES::

            sage: K = delta_complexes.KleinBottle()
            sage: K.barycentric_subdivision()
            Traceback (most recent call last):
            ...
            NotImplementedError: barycentric subdivisions are not implemented for Delta complexes
        """
        raise NotImplementedError("barycentric subdivisions are not implemented for Delta complexes")

    def n_chains(self, n, base_ring=None, cochains=False):
        r"""
        Return the free module of chains in degree ``n`` over ``base_ring``.

        INPUT:

        - ``n`` -- integer
        - ``base_ring`` -- ring (default: `\ZZ`)
        - ``cochains`` -- boolean (default: ``False``); if
          ``True``, return cochains instead

        Since the list of `n`-cells for a `\Delta`-complex may have
        some ambiguity -- for example, the list of edges may look like
        ``[(0, 0), (0, 0), (0, 0)]`` if each edge starts and ends at
        vertex 0 -- we record the indices of the cells along with
        their tuples. So the basis of chains in such a case would look
        like ``[(0, (0, 0)), (1, (0, 0)), (2, (0, 0))]``.

        The only difference between chains and cochains is notation:
        the dual cochain to the chain basis element ``b`` is written
        as ``\chi_b``.

        EXAMPLES::

            sage: T = delta_complexes.Torus()
            sage: T.n_chains(1, QQ)                                                     # needs sage.modules
            Free module generated by {(0, (0, 0)), (1, (0, 0)), (2, (0, 0))}
             over Rational Field
            sage: list(T.n_chains(1, QQ, cochains=False).basis())                       # needs sage.modules
            [(0, (0, 0)), (1, (0, 0)), (2, (0, 0))]
            sage: list(T.n_chains(1, QQ, cochains=True).basis())                        # needs sage.modules
            [\chi_(0, (0, 0)), \chi_(1, (0, 0)), \chi_(2, (0, 0))]
        """
        from sage.homology.chains import Chains, Cochains

        n_cells = tuple(enumerate(self.n_cells(n)))
        if cochains:
            return Cochains(self, n, n_cells, base_ring)
        else:
            return Chains(self, n, n_cells, base_ring)

    # the second barycentric subdivision is a simplicial complex.  implement this somehow?
#     def simplicial_complex(self):
#         X = self.barycentric_subdivision().barycentric_subdivision()
#         find facets of X and return SimplicialComplex(facets)

    # This is cached for speed reasons: it can be very slow to run
    # this function.
    @cached_method
    def algebraic_topological_model(self, base_ring=None):
        r"""
        Algebraic topological model for this `\Delta`-complex with
        coefficients in ``base_ring``.

        The term "algebraic topological model" is defined by Pilarczyk
        and Réal [PR2015]_.

        INPUT:

        - ``base_ring`` -- coefficient ring (default: ``QQ``); must be a field

        Denote by `C` the chain complex associated to this
        `\Delta`-complex. The algebraic topological model is a chain complex
        `M` with zero differential, with the same homology as `C`,
        along with chain maps `\pi: C \to M` and `\iota: M \to C`
        satisfying `\iota \pi = 1_M` and `\pi \iota` chain homotopic
        to `1_C`. The chain homotopy `\phi` must satisfy

        - `\phi \phi = 0`,
        - `\pi \phi = 0`,
        - `\phi \iota = 0`.

        Such a chain homotopy is called a *chain contraction*.

        OUTPUT: a pair consisting of

        - chain contraction ``phi`` associated to `C`, `M`, `\pi`, and
          `\iota`
        - the chain complex `M`

        Note that from the chain contraction ``phi``, one can recover the
        chain maps `\pi` and `\iota` via ``phi.pi()`` and
        ``phi.iota()``. Then one can recover `C` and `M` from, for
        example, ``phi.pi().domain()`` and ``phi.pi().codomain()``,
        respectively.

        EXAMPLES::

            sage: # needs sage.modules
            sage: RP2 = delta_complexes.RealProjectivePlane()
            sage: phi, M = RP2.algebraic_topological_model(GF(2))
            sage: M.homology()
            {0: Vector space of dimension 1 over Finite Field of size 2,
             1: Vector space of dimension 1 over Finite Field of size 2,
             2: Vector space of dimension 1 over Finite Field of size 2}
            sage: T = delta_complexes.Torus()
            sage: phi, M = T.algebraic_topological_model(QQ)
            sage: M.homology()
            {0: Vector space of dimension 1 over Rational Field,
             1: Vector space of dimension 2 over Rational Field,
             2: Vector space of dimension 1 over Rational Field}
        """
        from sage.homology.algebraic_topological_model import algebraic_topological_model_delta_complex
        if base_ring is None:
            base_ring = QQ
        return algebraic_topological_model_delta_complex(self, base_ring)

    def _string_constants(self):
        r"""
        Tuple containing the name of the type of complex, and the
        singular and plural of the name of the cells from which it is
        built.  This is used in constructing the string representation.

        EXAMPLES::

            sage: T = delta_complexes.Torus()
            sage: T._string_constants()
            ('Delta', 'simplex', 'simplices')
        """
        return ('Delta', 'simplex', 'simplices')


class DeltaComplexExamples:
    r"""
    Some examples of `\Delta`-complexes.

    Here are the available examples; you can also type
    ``delta_complexes.``  and hit TAB to get a list::

        Sphere
        Torus
        RealProjectivePlane
        KleinBottle
        Simplex
        SurfaceOfGenus

    EXAMPLES::

        sage: S = delta_complexes.Sphere(6) # the 6-sphere
        sage: S.dimension()
        6
        sage: S.cohomology(6)                                                           # needs sage.modules
        Z
        sage: delta_complexes.Torus() == delta_complexes.Sphere(3)
        False
    """

    def Sphere(self, n):
        r"""
        A `\Delta`-complex representation of the `n`-dimensional sphere,
        formed by gluing two `n`-simplices along their boundary,
        except in dimension 1, in which case it is a single 1-simplex
        starting and ending at the same vertex.

        INPUT:

        - ``n`` -- dimension of the sphere

        EXAMPLES::

            sage: delta_complexes.Sphere(4).cohomology(4, base_ring=GF(3))              # needs sage.modules
            Vector space of dimension 1 over Finite Field of size 3
        """
        if n == 1:
            return DeltaComplex([[()], [(0, 0)]])
        return DeltaComplex({Simplex(n): True, Simplex(range(1, n+2)): Simplex(n)})

    def Torus(self):
        r"""
        A `\Delta`-complex representation of the torus, consisting of one
        vertex, three edges, and two triangles.

        .. image:: ../../media/torus.png

        EXAMPLES::

            sage: delta_complexes.Torus().homology(1)                                   # needs sage.modules sage.rings.finite_rings
            Z x Z
        """
        return DeltaComplex((((),), ((0, 0), (0, 0), (0, 0)),
                             ((1, 2, 0), (0, 2, 1))))

    def RealProjectivePlane(self):
        r"""
        A `\Delta`-complex representation of the real projective plane,
        consisting of two vertices, three edges, and two triangles.

        .. image:: ../../media/rp2.png

        EXAMPLES::

            sage: # needs sage.modules
            sage: P = delta_complexes.RealProjectivePlane()
            sage: P.cohomology(1)
            0
            sage: P.cohomology(2)
            C2
            sage: P.cohomology(dim=1, base_ring=GF(2))
            Vector space of dimension 1 over Finite Field of size 2
            sage: P.cohomology(dim=2, base_ring=GF(2))
            Vector space of dimension 1 over Finite Field of size 2
        """
        return DeltaComplex((((), ()), ((1, 0), (1, 0), (0, 0)),
                             ((1, 0, 2), (0, 1, 2))))

    def KleinBottle(self):
        r"""
        A `\Delta`-complex representation of the Klein bottle, consisting
        of one vertex, three edges, and two triangles.

        .. image:: ../../media/klein.png

        EXAMPLES::

            sage: delta_complexes.KleinBottle()
            Delta complex with 1 vertex and 7 simplices
        """
        return DeltaComplex((((),), ((0, 0), (0, 0), (0, 0)),
                             ((1, 2, 0), (0, 1, 2))))

    def Simplex(self, n):
        r"""
        A `\Delta`-complex representation of an `n`-simplex,
        consisting of a single `n`-simplex and its faces.  (This is
        the same as the simplicial complex representation available by
        using ``simplicial_complexes.Simplex(n)``.)

        EXAMPLES::

            sage: delta_complexes.Simplex(3)
            Delta complex with 4 vertices and 16 simplices
        """
        return DeltaComplex({Simplex(n): True})

    def SurfaceOfGenus(self, g, orientable=True):
        r"""
        A surface of genus g as a `\Delta`-complex.

        INPUT:

        - ``g`` -- nonnegative integer; the genus
        - ``orientable`` -- boolean (default: ``True``); whether the surface
          should be orientable

        In the orientable case, return a sphere if `g` is zero, and
        otherwise return a `g`-fold connected sum of a torus with
        itself.

        In the non-orientable case, raise an error if `g` is zero.  If
        `g` is positive, return a `g`-fold connected sum of a
        real projective plane with itself.

        EXAMPLES::

            sage: delta_complexes.SurfaceOfGenus(1, orientable=False)
            Delta complex with 2 vertices and 8 simplices
            sage: delta_complexes.SurfaceOfGenus(3, orientable=False).homology(1)       # needs sage.modules
            Z x Z x C2
            sage: delta_complexes.SurfaceOfGenus(3, orientable=False).homology(2)       # needs sage.modules
            0

        Compare to simplicial complexes::

            sage: delta_g4 = delta_complexes.SurfaceOfGenus(4)
            sage: delta_g4.f_vector()
            [1, 3, 27, 18]
            sage: simpl_g4 = simplicial_complexes.SurfaceOfGenus(4)
            sage: simpl_g4.f_vector()
            [1, 19, 75, 50]
            sage: delta_g4.homology() == simpl_g4.homology()                            # needs sage.modules
            True
        """
        try:
            g = Integer(g)
        except TypeError:
            raise ValueError("genus must be a nonnegative integer")
        if g < 0:
            raise ValueError("genus must be a nonnegative integer")
        if g == 0:
            if not orientable:
                raise ValueError("no non-orientable surface of genus zero")
            else:
                return delta_complexes.Sphere(2)
        if orientable:
            X = delta_complexes.Torus()
        else:
            X = delta_complexes.RealProjectivePlane()
        S = X
        for i in range(g - 1):
            S = S.connected_sum(X)
        return S


delta_complexes = DeltaComplexExamples()



# sage_setup: distribution = sagemath-graphs
# cython: binding=True
r"""
Products of graphs

This module gathers everything related to graph products. At the moment it
contains an implementation of a recognition algorithm for graphs that can be
written as a Cartesian product of smaller ones.

Author:

- Nathann Cohen (May 2012 -- coded while watching the election of Francois
  Hollande on TV)

Cartesian product of graphs -- the recognition problem
------------------------------------------------------

First, a definition:

  **Definition** The Cartesian product of two graphs `G` and `H`, denoted
  `G\square H`, is a graph defined on the pairs `(g, h)\in V(G)\times V(H)`.

  Two elements `(g, h),(g', h')\in V(G\square H)` are adjacent in `G\square H`
  if and only if :

  - `g=g'` and `hh'\in H`; or
  - `h=h'` and `gg'\in G`

Two remarks follow :

#. The Cartesian product is commutative

#. Any edge `uv` of a graph `G_1 \square \cdots \square G_k` can be given a
   color `i` corresponding to the unique index `i` such that `u_i` and `v_i`
   differ.

The problem that is of interest to us in the present module is the following:

  **Recognition problem** Given a graph `G`, can we guess whether there exist
  graphs `G_1, ..., G_k` such that `G=G_1\square \cdots \square G_k` ?

This problem can actually be solved, and the resulting factorization is
unique. What is explained below can be found in the book *Handbook of Product
Graphs* [HIK2011]_.

Everything is actually based on simple observations. Given a graph `G`, finding
out whether `G` can be written as the product of several graphs can be attempted
by trying to color its edges according to some rules. Indeed, if we are to color
the edges of `G` in such a way that each color class represents a factor of `G`,
we must ensure several things.

  **Remark 1** In any cycle of `G` no color can appear exactly once.

  Indeed, if only one edge `uv` of a cycle were labelled with color `i`, it
  would mean that:

  #. The only difference between `u` and `v` lies in their `i` th coordinate

  #. It is possible to go from `u` to `v` by changing only coordinates
     different from the `i` th

  A contradiction indeed.

  .. image:: ../../../media/cycle.png

  That means that, for instance, the edges of a triangle necessarily have the
  same color.

  **Remark 2** If two consecutive edges `u_1u_2` and `u_2u_3` have different
  colors, there necessarily exists a unique vertex `u_4` different from `u_2`
  and incident to both `u_1` and `u_3`.

  In this situation, opposed edges necessarily have the same colors because of
  the previous remark.

  .. image:: ../../../media/square.png

  **1st criterion** : As a corollary, we know that:

  #. If two vertices `u,v` have a *unique* common neighbor `x`, then `ux` and
     `xv` have the same color.

  #. If two vertices `u, v` have more that two common neighbors `x_1, ...,
     x_k` then all edges between the `x_i` and the vertices of `u,v` have the
     same color. This is also a consequence of the first remark.

  **2nd criterion** : if two edges `uv` and `u'v'` of the product graph
  `G\square H` are such that `d(u,u')+d(v,v')\neq d(u,v') + d(v,u')` then the
  two edges `uv` and `u'v'` necessarily have the same color.

    This is a consequence of the fact that for any two vertices `u,v` of
    `G\square H` (where `u=(u_G,u_H)` and `v=(v_G,v_H)`), we have `d(u,v) =
    d_G(u_G,v_G)+d_H(u_H,v_H)`. Indeed, a shortest path from `u` to `v` in
    `G\square H` contains the information of a shortest path from `u_G` to `v_G`
    in `G`, and a shortest path from `u_H` to `v_H` in `H`.

The algorithm
^^^^^^^^^^^^^

The previous remarks tell us that some edges are in some way equivalent to some
others, i.e. that their colors are equal. In order to compute the coloring we
are looking for, we therefore build a graph on the *edges* of a graph `G`,
linking two edges whenever they are found to be equivalent according to the
previous remarks.

All that is left to do is to compute the connected components of this new graph,
as each of them representing the edges of a factor. Of course, only one
connected component indicates that the graph has no factorization.

Then again, please refer to [HIK2011]_ for any technical question.

To Do
^^^^^

This implementation is made at Python level, and some parts of the algorithm
could be rewritten in Cython to save time. Especially when enumerating all pairs
of edges and computing their distances. This can easily be done in C with the
functions from the :mod:`sage.graphs.distances_all_pairs` module.

Methods
-------
"""

# ****************************************************************************
#       Copyright (C) 2012 Nathann Cohen <nathann.cohen@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************


def is_cartesian_product(g, certificate=False, relabeling=False):
    r"""
    Test whether the graph is a Cartesian product.

    INPUT:

    - ``certificate`` -- boolean (default: ``False``); if ``certificate =
      False`` (default) the method only returns ``True`` or ``False``
      answers. If ``certificate = True``, the ``True`` answers are replaced by
      the list of the factors of the graph.

    - ``relabeling`` -- boolean (default: ``False``); if ``relabeling = True``
      (implies ``certificate = True``), the method also returns a dictionary
      associating to each vertex its natural coordinates as a vertex of a
      product graph. If `g` is not a Cartesian product, ``None`` is returned
      instead.

    .. SEEALSO::

        - :meth:`sage.graphs.generic_graph.GenericGraph.cartesian_product`

        - :mod:`~sage.graphs.graph_decompositions.graph_products` -- a module on
          graph products

    .. NOTE::

        This algorithm may run faster whenever the graph's vertices are integers
        (see :meth:`~sage.graphs.generic_graph.GenericGraph.relabel`). Give it a
        try if it is too slow !

    EXAMPLES:

    The Petersen graph is prime::

        sage: from sage.graphs.graph_decompositions.graph_products import is_cartesian_product
        sage: g = graphs.PetersenGraph()
        sage: is_cartesian_product(g)
        False

    A 2d grid is the product of paths::

        sage: g = graphs.Grid2dGraph(5,5)
        sage: p1, p2 = is_cartesian_product(g, certificate = True)
        sage: p1.is_isomorphic(graphs.PathGraph(5))
        True
        sage: p2.is_isomorphic(graphs.PathGraph(5))
        True

    Forgetting the graph's labels, then finding them back::

        sage: g.relabel()
        sage: b,D = g.is_cartesian_product(g, relabeling=True)
        sage: b
        True
        sage: D  # random isomorphism
        {0: (20, 0), 1: (20, 1), 2: (20, 2), 3: (20, 3), 4: (20, 4),
         5: (15, 0), 6: (15, 1), 7: (15, 2), 8: (15, 3), 9: (15, 4),
         10: (10, 0), 11: (10, 1), 12: (10, 2), 13: (10, 3), 14: (10, 4),
         15: (5, 0), 16: (5, 1), 17: (5, 2), 18: (5, 3), 19: (5, 4),
         20: (0, 0), 21: (0, 1), 22: (0, 2), 23: (0, 3), 24: (0, 4)}

    And of course, we find the factors back when we build a graph from a
    product::

        sage: g = graphs.PetersenGraph().cartesian_product(graphs.CycleGraph(3))
        sage: g1, g2 = is_cartesian_product(g, certificate = True)
        sage: any( x.is_isomorphic(graphs.PetersenGraph()) for x in [g1,g2])
        True
        sage: any( x.is_isomorphic(graphs.CycleGraph(3)) for x in [g1,g2])
        True

    TESTS:

    Wagner's Graph (:issue:`13599`)::

        sage: g = graphs.WagnerGraph()                                                  # needs networkx
        sage: g.is_cartesian_product()                                                  # needs networkx
        False

    Empty and one-element graph (:issue:`19546`)::

        sage: Graph().is_cartesian_product()
        False
        sage: Graph({0:[]}).is_cartesian_product()
        False
    """
    g._scream_if_not_simple()
    if g.is_directed():
        raise NotImplementedError("recognition of Cartesian product is not implemented for directed graphs")
    if relabeling:
        certificate = True

    from sage.rings.integer import Integer

    if not g.is_connected():
        raise NotImplementedError("recognition of Cartesian product is not implemented for disconnected graphs")

    # Of course the number of vertices of g cannot be prime !
    if g.order() <= 3 or Integer(g.order()).is_prime():
        return (False, None) if relabeling else False

    from sage.graphs.graph import Graph

    # As we need the vertices of g to be linearly ordered, we copy the graph and
    # relabel it
    cdef list int_to_vertex = list(g)
    cdef dict vertex_to_int = {vert: i for i, vert in enumerate(int_to_vertex)}
    g_int = g.relabel(perm=vertex_to_int, inplace=False)

    # Reorder the vertices of an edge
    def r(x, y):
        return (x, y) if x < y else (y, x)

    cdef int x, y, u, v
    cdef set un, intersect

    # The equivalence graph on the edges of g
    h = Graph()
    h.add_vertices(r(x, y) for x, y in g_int.edge_iterator(labels=False))

    # For all pairs of vertices u,v of G, according to their number of common
    # neighbors... See the module's documentation !
    for u in g_int:
        un = set(g_int.neighbor_iterator(u))
        for v in g_int.breadth_first_search(u):

            # u and v are different
            if u == v:
                continue

            # List of common neighbors
            intersect = un & set(g_int.neighbor_iterator(v))

            # If u and v have no neighbors and uv is not an edge then their
            # distance is at least 3. As we enumerate the vertices in a
            # breadth-first search, it means that we already checked all the
            # vertices at distance less than two from u, and we are done with
            # this loop !
            if not intersect:
                if g_int.has_edge(u, v):
                    continue
                else:
                    break

            # If uv is an edge
            if g_int.has_edge(u, v):
                h.add_path([r(u, x) for x in intersect] + [r(v, x) for x in intersect])

            # Only one common neighbor
            elif len(intersect) == 1:
                x = intersect.pop()
                h.add_edge(r(u, x), r(v, x))

            # Exactly 2 neighbors
            elif len(intersect) == 2:
                x, y = intersect
                h.add_edge(r(u, x), r(v, y))
                h.add_edge(r(v, x), r(u, y))
            # More
            else:
                h.add_path([r(u, x) for x in intersect] + [r(v, x) for x in intersect])

    # Edges uv and u'v' such that d(u,u')+d(v,v') != d(u,v')+d(v,u') are also
    # equivalent

    cdef list edges = list(g_int.edges(labels=False, sort=False))
    cdef dict d = g_int.distance_all_pairs()
    cdef int uu, vv
    for i, (u, v) in enumerate(edges):
        du = d[u]
        dv = d[v]
        for j in range(i + 1, g_int.size()):
            uu, vv = edges[j]
            if du[uu] + dv[vv] != du[vv] + dv[uu]:
                h.add_edge(r(u, v), r(uu, vv))

    # Gathering the connected components, relabeling the vertices on-the-fly
    edges = [[(int_to_vertex[u], int_to_vertex[v]) for u, v in cc]
             for cc in h.connected_components(sort=False)]

    # Only one connected component ?
    if len(edges) == 1:
        return (False, None) if relabeling else False

    # Building the list of factors
    cdef list factors = []
    for cc in edges:
        tmp = Graph()
        tmp.add_edges(cc)
        factors.append(tmp.subgraph(vertices=tmp.connected_components(sort=False)[0]))

    # Computing the product of these graphs
    answer = factors[0]
    for i in range(1, len(factors)):
        answer = answer.cartesian_product(factors[i])

    # Checking that the resulting graph is indeed isomorphic to what we have.
    isiso, dictt = g.is_isomorphic(answer, certificate=True)
    if not isiso:
        raise ValueError("something weird happened during the algorithm... "
                         "Please report the bug and give us the graph instance"
                         " that made it fail !")
    if relabeling:
        return isiso, dictt
    if certificate:
        return factors
    else:
        return True


def rooted_product(G, H, root=None):
    r"""
    Return the rooted product of `G` and `H`.

    The rooted product of two graphs `G` and `H` is the graph `R` defined as
    follows: take a copy of `G` and `|V(G)|` copies of `H`, and for every vertex
    `g_i` of `G`, identify `g_i` with the root of the `i`-th copy of `H`.
    Mode formally, let `V(G) = \{g_1, g_2, \ldots, g_n\}`,
    `V(H) = \{h_1, h_2, \ldots, h_m\}`, and let `h_1` be the root vertex of `H`.
    The vertex set `V(R)` is equal to the cartesian product of the sets of
    vertices `V(G)` and `V(H)`, that is
    `V(R) = \{(g_i, h_j) : g_i \in V(G), h_j \in V(H)\}`.  The edge set `E(R)`
    is the union of the edges of a copy of `G`, that is
    `\{((g_i, h_1), (g_j, h_1)) : (g_i, g_j) \in E(G)\}`, and the edges of the
    copies of `H` for every `g_i \in V(G)`, that is
    `\{((g_i, h_j), (g_i, h_k)) : (h_j, h_k) \in V(H)\}`.

    See :wikipedia:`Rooted_product_of_graphs` for more details.

    .. SEEALSO::

        - :meth:`~sage.graphs.generic_graph.cartesian_product`
          -- return the cartesian product of two graphs

        - :mod:`~sage.graphs.graph_decompositions.graph_products`
          -- a module on graph products

    EXAMPLES:

    The rooted product of two trees is a tree::

        sage: T1 = graphs.RandomTree(7)
        sage: T2 = graphs.RandomTree(8)
        sage: T = T1.rooted_product(T2)
        sage: T.is_tree()
        True

    The rooted product of `G` and `H` depends on the selected root in `H`::

        sage: G = graphs.CycleGraph(4)
        sage: H = graphs.PathGraph(3)
        sage: R1 = G.rooted_product(H, root=0)
        sage: R2 = G.rooted_product(H, root=1)
        sage: R1.is_isomorphic(R2)
        False
        sage: sorted(R1.degree())
        [1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3]
        sage: sorted(R2.degree())
        [1, 1, 1, 1, 1, 1, 1, 1, 4, 4, 4, 4]

    The domination number of the rooted product of any graph `G` and a path of
    order 2 is the order of `G`::

        sage: G = graphs.RandomGNP(20, .3)
        sage: P = graphs.PathGraph(2)
        sage: R = G.rooted_product(P)
        sage: len(R.dominating_set()) == G.order()                                      # needs sage.numerical.mip
        True
        sage: G = digraphs.RandomDirectedGNP(20, .3)
        sage: P = digraphs.Path(2)
        sage: R = G.rooted_product(P)
        sage: len(R.dominating_set()) == G.order()                                      # needs sage.numerical.mip
        True

    The rooted product of two graphs is a subgraph of the cartesian product of
    the same two graphs::

        sage: G = graphs.RandomGNP(6, .4)
        sage: H = graphs.RandomGNP(7, .4)
        sage: R = G.rooted_product(H)
        sage: C = G.cartesian_product(H)
        sage: R.is_subgraph(C, induced=False)
        True

    Corner cases::

        sage: Graph().rooted_product(Graph())
        Rooted product of Graph on 0 vertices and Graph on 0 vertices: Graph on 0 vertices
        sage: Graph(1).rooted_product(Graph())
        Rooted product of Graph on 1 vertex and Graph on 0 vertices: Graph on 0 vertices
        sage: Graph().rooted_product(Graph(1))
        Rooted product of Graph on 0 vertices and Graph on 1 vertex: Graph on 0 vertices
        sage: Graph(1).rooted_product(Graph(1))
        Rooted product of Graph on 1 vertex and Graph on 1 vertex: Graph on 1 vertex

    TESTS::

        sage: Graph().rooted_product(DiGraph())
        Traceback (most recent call last):
        ...
        TypeError: the graphs should be both directed or both undirected
    """
    G._scream_if_not_simple(allow_loops=True)
    if G._directed and H._directed:
        from sage.graphs.digraph import DiGraph
        R = DiGraph(loops=(G.has_loops() or H.has_loops()))
    elif (not G._directed) and (not H._directed):
        from sage.graphs.graph import Graph
        R = Graph(loops=(G.has_loops() or H.has_loops()))
    else:
        raise TypeError('the graphs should be both directed or both undirected')

    R.name(f'Rooted product of {G} and {H}')

    if not G or not H:
        return R
    if root is None:
        root = next(H.vertex_iterator())
    elif root not in H:
        raise ValueError("the specified root is not a vertex of H")

    R.add_vertices((u, x) for u in G for x in H)
    for u, v in G.edge_iterator(labels=False):
        R.add_edge((u, root), (v, root))
    for x, y in H.edge_iterator(labels=False):
        R.add_edges(((u, x), (u, y)) for u in G)

    return R



# sage_setup: distribution = sagemath-graphs
r"""
Fast dense graphs

For an overview of graph data structures in sage, see
:mod:`~sage.graphs.base.overview`.

Usage Introduction
------------------

::

    sage: from sage.graphs.base.dense_graph import DenseGraph

Dense graphs are initialized as follows::

    sage: D = DenseGraph(nverts=10, extra_vertices=10)

This example initializes a dense graph with room for twenty vertices, the first
ten of which are in the graph. In general, the first ``nverts`` are "active."
For example, see that 9 is already in the graph::

    sage: D.verts()
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
    sage: D.add_vertex(9)
    9
    sage: D.verts()
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

But 10 is not, until we add it::

    sage: D.add_vertex(10)
    10
    sage: D.verts()
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]

You can begin working right away as follows::

    sage: D.add_arc(0, 1)
    sage: D.add_arc(1, 2)
    sage: D.add_arc(1, 0)
    sage: D.has_arc(7, 3)
    False
    sage: D.has_arc(0, 1)
    True
    sage: D.in_neighbors(1)
    [0]
    sage: D.out_neighbors(1)
    [0, 2]
    sage: D.del_all_arcs(0, 1)
    sage: D.has_arc(0, 1)
    False
    sage: D.has_arc(1, 2)
    True
    sage: D.del_vertex(7)
    sage: D.has_arc(7, 3)
    False

Dense graphs do not support multiple or labeled edges.

::

    sage: T = DenseGraph(nverts=3, extra_vertices=2)
    sage: T.add_arc(0, 1)
    sage: T.add_arc(1, 2)
    sage: T.add_arc(2, 0)
    sage: T.has_arc(0, 1)
    True

::

    sage: for _ in range(10): D.add_arc(5, 4)
    sage: D.has_arc(5,4 )
    True

Dense graphs are by their nature directed. As of this writing, you need to do
operations in pairs to treat the undirected case (or use a backend or a Sage
graph)::

    sage: T.has_arc(1, 0)
    False

The curious developer is encouraged to check out the ``unsafe`` functions,
which do not check input but which run in pure C.

Underlying Data Structure
-------------------------

The class ``DenseGraph`` contains the following variables which are inherited
from ``CGraph`` (for explanation, refer to the documentation there)::

        cdef int num_verts
        cdef int num_arcs
        cdef int *in_degrees
        cdef int *out_degrees
        cdef bitset_t active_vertices

It also contains the following variables::

        cdef binary_matrix_t edges

.. NOTE::

    As the edges are stored as the adjacency matrix of the graph, enumerating
    the edges of the graph has complexity `O(n^2)` and enumerating the neighbors
    of a vertex has complexity `O(n)` (where `n` in the size of the bitset
    active_vertices).
    So, the class ``DenseGraph`` should be used for graphs such that the number
    of edges is close to the square of the number of vertices.

"""

# ****************************************************************************
#       Copyright (C) 2008-9 Robert L. Miller <rlmillster@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.data_structures.bitset_base cimport *

from cysignals.memory cimport sig_calloc, sig_realloc, sig_free
from sage.data_structures.binary_matrix cimport *
from libc.string cimport memcpy


cdef extern from "Python.h":
    int unlikely(int) nogil  # Defined by Cython


cdef class DenseGraph(CGraph):
    """
    Compiled dense graphs.

    ::

        sage: from sage.graphs.base.dense_graph import DenseGraph

    Dense graphs are initialized as follows::

        sage: D = DenseGraph(nverts=10, extra_vertices=10)

    INPUT:

    - ``nverts`` -- nonnegative integer; the number of vertices
    - ``extra_vertices`` -- nonnegative integer (default: 10); how many extra
      vertices to allocate
    - ``verts`` -- list (default: ``None``); optional list of vertices to add
    - ``arcs`` -- list (default: ``None``); optional list of arcs to add
    - ``directed`` -- boolean (default: ``None``); whether the graph is directed

    The first ``nverts`` are created as vertices of the graph, and the next
    ``extra_vertices`` can be freely added without reallocation. See top level
    documentation for more details. The input ``verts`` and ``arcs`` are mainly
    for use in pickling.
    """
    def __cinit__(self, int nverts, int extra_vertices=10, verts=None, arcs=None, directed=True):
        """
        Allocation and initialization happen in one place.

        Memory usage is ``O((nverts + extra_vertices)^2)``.

        EXAMPLES::

            sage: from sage.graphs.base.dense_graph import DenseGraph
            sage: D = DenseGraph(nverts=10, extra_vertices=10)
        """
        if not nverts and not extra_vertices:
            raise RuntimeError('dense graphs must allocate space for vertices')

        self.num_verts = nverts
        self.num_arcs = 0
        cdef int total_verts = nverts + extra_vertices
        self._directed = directed

        binary_matrix_init(self.edges, total_verts, total_verts)
        self.in_degrees = <int *> sig_calloc(total_verts, sizeof(int))
        self.out_degrees = <int *> sig_calloc(total_verts, sizeof(int))

        if not self.in_degrees or not self.out_degrees:
            sig_free(self.in_degrees)
            sig_free(self.out_degrees)
            binary_matrix_free(self.edges)
            raise MemoryError

        bitset_init(self.active_vertices, total_verts)
        bitset_set_first_n(self.active_vertices, self.num_verts)

        if verts is not None:
            self.add_vertices(verts)

        if arcs is not None:
            for u, v in arcs:
                self.add_arc(u, v)

    def __dealloc__(self):
        """
        New and dealloc are both tested at class level.
        """
        binary_matrix_free(self.edges)
        sig_free(self.in_degrees)
        sig_free(self.out_degrees)
        bitset_free(self.active_vertices)

    cpdef realloc(self, int total_verts):
        """
        Reallocate the number of vertices to use, without actually adding any.

        INPUT:

        - ``total`` -- integer; the total size to make the array

        Returns -1 and fails if reallocation would destroy any active vertices.

        EXAMPLES::

            sage: from sage.graphs.base.dense_graph import DenseGraph
            sage: D = DenseGraph(nverts=4, extra_vertices=4)
            sage: D.current_allocation()
            8
            sage: D.add_vertex(6)
            6
            sage: D.current_allocation()
            8
            sage: D.add_vertex(10)
            10
            sage: D.current_allocation()
            16
            sage: D.add_vertex(40)
            Traceback (most recent call last):
            ...
            RuntimeError: requested vertex is past twice the allocated range: use realloc
            sage: D.realloc(50)
            sage: D.add_vertex(40)
            40
            sage: D.current_allocation()
            50
            sage: D.realloc(30)
            -1
            sage: D.current_allocation()
            50
            sage: D.del_vertex(40)
            sage: D.realloc(30)
            sage: D.current_allocation()
            30
        """
        cdef int i
        if not total_verts:
            raise RuntimeError('dense graphs must allocate space for vertices')

        cdef bitset_t bits
        cdef int min_verts
        if <size_t>total_verts < self.active_vertices.size:
            min_verts = total_verts
            bitset_init(bits, self.active_vertices.size)
            bitset_set_first_n(bits, total_verts)
            if not bitset_issubset(self.active_vertices, bits):
                bitset_free(bits)
                return -1
            bitset_free(bits)
        else:
            min_verts = self.active_vertices.size

        # Resize of self.edges
        binary_matrix_realloc(self.edges, total_verts, total_verts)

        self.in_degrees = <int *> sig_realloc(self.in_degrees, total_verts * sizeof(int))
        self.out_degrees = <int *> sig_realloc(self.out_degrees, total_verts * sizeof(int))

        for i in range(self.active_vertices.size, total_verts):
            self.in_degrees[i] = 0
            self.out_degrees[i] = 0

        bitset_realloc(self.active_vertices, total_verts)

    ###################################
    # Arc functions
    ###################################

    cdef inline int _add_arc_unsafe(self, int u, int v) except -1:
        """
        .. WARNING::

            This method is for internal use only. Use :meth:`add_arc_label_unsafe` instead.

        Add arc (u, v) with label l in only one direction.
        """
        if not binary_matrix_get(self.edges, u, v):
            self.in_degrees[v] += 1
            self.out_degrees[u] += 1
            self.num_arcs += 1
            binary_matrix_set1(self.edges, u, v)

    cdef int add_arc_label_unsafe(self, int u, int v, int l) except -1:
        """
        Add arc ``(u, v)`` to the graph.

        INPUT:

        - ``u``, ``v`` -- nonnegative integers
        """
        if unlikely(l):
            raise ValueError("cannot add a labeled arc to an unlabeled graph")

        self._add_arc_unsafe(u, v)
        if u != v and not self._directed:
            self._add_arc_unsafe(v, u)

    cdef int has_arc_label_unsafe(self, int u, int v, int l) except -1:
        """
        Check whether arc ``(u, v)`` is in the graph.

        INPUT:

        - ``u``, ``v`` -- nonnegative integers, must be in self

        - ``l`` -- positive integer label, or zero for no label, or ``-1`` for any label

        OUTPUT:

        - 0 -- False
        - 1 -- True
        """
        if unlikely(l > 0):
            raise ValueError("cannot locate labeled arc in unlabeled graph")
        return binary_matrix_get(self.edges, u, v)

    cdef inline int _del_arc_unsafe(self, int u, int v) except -1:
        """
        .. WARNING::

            This method is for internal use only. Use :meth:`add_arc_label_unsafe` instead.

        Remove arc (u, v) with label l in only one direction.
        """
        if binary_matrix_get(self.edges, u, v):
            self.in_degrees[v] -= 1
            self.out_degrees[u] -= 1
            self.num_arcs -= 1
            binary_matrix_set0(self.edges, u, v)

    cdef int del_arc_unsafe(self, int u, int v) except -1:
        """
        Delete the arc from ``u`` to ``v``, if it exists.

        INPUT:

        - ``u``, ``v`` -- nonnegative integers, must be in self
        """
        self._del_arc_unsafe(u, v)
        if u != v and not self._directed:
            self._del_arc_unsafe(v, u)

    cdef inline int del_arc_label_unsafe(self, int u, int v, int l) except -1:
        if unlikely(l):
            raise ValueError("cannot delete labeled arc in unlabeled graph")
        return self.del_arc_unsafe(u, v)

    cdef inline int arc_label_unsafe(self, int u, int v) except -1:
        return 0

    cdef int all_arcs_unsafe(self, int u, int v, int* labels, int size) except -1:
        """
        Give the labels of all arcs (u, v).

        INPUT:

        - ``u``, ``v`` -- integers from `0`, ..., `n-1`, where `n` is the
          number of vertices
        - ``labels`` -- must be a pointer to an (allocated) integer array
        - ``size`` -- the length of the array

        OUTPUT: integer; the number of arcs ``(u, v)``.
        ``-1`` -- indicates that the array has been filled with labels, but
        there were more.
        """
        if self.has_arc_unsafe(u, v):
            if size > 0:
                labels[0] = 0
                return 1
            else:
                return -1
        return 0

    def complement(self):
        r"""
        Replace the graph with its complement.

        .. NOTE::

            Assumes that the graph has no loop.

        EXAMPLES::

            sage: from sage.graphs.base.dense_graph import DenseGraph
            sage: G = DenseGraph(5)
            sage: G.add_arc(0, 1)
            sage: G.has_arc(0, 1)
            True
            sage: G.complement()
            sage: G.has_arc(0, 1)
            False
        """
        cdef int num_arcs_old = self.num_arcs

        cdef size_t i
        i = bitset_next(self.active_vertices, 0)
        while i != -1:
            self.add_arc_unsafe(i, i)
            bitset_xor(self.edges.rows[i], self.edges.rows[i], self.active_vertices)
            self.in_degrees[i] = self.num_verts-self.in_degrees[i]
            self.out_degrees[i] = self.num_verts-self.out_degrees[i]
            i = bitset_next(self.active_vertices, i + 1)

        self.num_arcs = self.num_verts*(self.num_verts - 1) - num_arcs_old

    ###################################
    # Neighbor functions
    ###################################

    cdef inline int next_out_neighbor_unsafe(self, int u, int v, int* l) except -2:
        """
        Return the next out-neighbor of ``u`` that is greater than ``v``.

        If ``v`` is ``-1`` return the first neighbor of ``u``.

        Return ``-1`` in case there does not exist such an out-neighbor.

        Set ``l`` to be the label of the first arc.
        """
        l[0] = 0
        return bitset_next(self.edges.rows[u], v + 1)

    cdef inline int next_in_neighbor_unsafe(self, int v, int u, int* l) except -2:
        """
        Return the next in-neighbor of ``v`` that is greater or equal to ``u``.

        If ``u`` is ``-1`` return the first neighbor of ``v``.

        Return ``-1`` in case there does not exist such a in-neighbor.

        Set ``l`` to be the label of the first arc.
        """
        l[0] = 0
        cdef size_t i
        i = bitset_next(self.active_vertices, u + 1)
        while i != -1:
            if binary_matrix_get(self.edges, i, v):
                return i
            i = bitset_next(self.active_vertices, i + 1)
        return -1


cdef int copy_dense_graph(DenseGraph dest, DenseGraph src) except -1:
    r"""
    Unsafely copy ``dest`` over ``src``.

    .. NOTE::

        ``dest.active_vertices`` and ``src.active_vertices`` must be of same size!
    """
    if unlikely(dest.active_vertices.size != src.active_vertices.size):
        raise ValueError("``dest.active_vertices`` and ``src.active_vertices`` must be of same size")
    if unlikely(dest.edges.n_rows != src.edges.n_rows or dest.edges.n_cols != src.edges.n_cols):
        raise ValueError("the edges are not of same size")
    memcpy(dest.in_degrees,  src.in_degrees,  src.active_vertices.size * sizeof(int))
    memcpy(dest.out_degrees, src.out_degrees, src.active_vertices.size * sizeof(int))
    binary_matrix_copy(dest.edges, src.edges)
    bitset_copy(dest.active_vertices, src.active_vertices)
    dest.num_verts = src.num_verts
    dest.num_arcs = src.num_arcs


##############################
# Further tests. Unit tests for methods, functions, classes defined with cdef.
##############################

def _test_adjacency_sequence_out():
    """
    Randomly test the method ``DenseGraph.adjacency_sequence_out()``. No output
    indicates that no errors were found.

    TESTS::

        sage: from sage.graphs.base.dense_graph import _test_adjacency_sequence_out
        sage: _test_adjacency_sequence_out()  # long time
    """
    from sage.graphs.digraph import DiGraph
    from sage.graphs.graph_generators import GraphGenerators
    from sage.misc.prandom import randint, random
    low = 0
    high = 500
    randg = DiGraph(GraphGenerators().RandomGNP(randint(low, high), random()))
    n = randg.order()
    while n <= 1:
        randg = DiGraph(GraphGenerators().RandomGNP(randint(low, high), random()))
        n = randg.order()

    cdef DenseGraph g = DenseGraph(n,
                                   verts=randg.vertex_iterator(),
                                   arcs=randg.edge_iterator(labels=False))
    assert g.num_verts == randg.order(), (
        "Graph order mismatch: %s vs. %s" % (g.num_verts, randg.order()))
    assert g.num_arcs == randg.size(), (
        "Graph size mismatch: %s vs. %s" % (g.num_arcs, randg.size()))
    M = randg.adjacency_matrix()
    cdef int *V = <int *>sig_malloc(n * sizeof(int))
    cdef int i = 0
    for v in randg.vertex_iterator():
        V[i] = v
        i += 1
    cdef int *seq = <int *> sig_malloc(n * sizeof(int))
    for i in range(randint(50, 101)):
        u = randint(low, n - 1)
        g.adjacency_sequence_out(n, V, u, seq)
        A = [seq[k] for k in range(n)]
        try:
            assert A == list(M[u])
        except AssertionError:
            sig_free(V)
            sig_free(seq)
            raise AssertionError("graph adjacency mismatch")
    sig_free(seq)
    sig_free(V)


###########################################
# Dense Graph Backend
###########################################

cdef class DenseGraphBackend(CGraphBackend):
    """
    Backend for Sage graphs using DenseGraphs.

    ::

        sage: from sage.graphs.base.dense_graph import DenseGraphBackend

    This class is only intended for use by the Sage Graph and DiGraph class.
    If you are interested in using a DenseGraph, you probably want to do
    something like the following example, which creates a Sage Graph instance
    which wraps a ``DenseGraph`` object::

        sage: G = Graph(30, sparse=False)
        sage: G.add_edges([(0, 1), (0, 3), (4, 5), (9, 23)])
        sage: G.edges(sort=True, labels=False)
        [(0, 1), (0, 3), (4, 5), (9, 23)]

    Note that Sage graphs using the backend are more flexible than DenseGraphs
    themselves. This is because DenseGraphs (by design) do not deal with Python
    objects::

        sage: G.add_vertex((0, 1, 2))
        sage: sorted(list(G),
        ....:        key=lambda x: (isinstance(x, tuple), x))
        [0,
        ...
         29,
         (0, 1, 2)]
        sage: from sage.graphs.base.dense_graph import DenseGraph
        sage: DG = DenseGraph(30)
        sage: DG.add_vertex((0, 1, 2))
        Traceback (most recent call last):
        ...
        TypeError: an integer is required
    """

    def __init__(self, n, directed=True):
        """
        Initialize a dense graph with ``n`` vertices.

        EXAMPLES::

            sage: D = sage.graphs.base.dense_graph.DenseGraphBackend(9)
            sage: D.add_edge(0, 1, None, False)
            sage: list(D.iterator_edges(range(9), True))
            [(0, 1, None)]
        """
        self._cg = DenseGraph(n, directed=directed)
        self._directed = directed
        self.vertex_labels = {}
        self.vertex_ints = {}

    cdef bint _delete_edge_before_adding(self) noexcept:
        """
        Return whether we should delete edges before adding any.

        As dense graphs do not support multiple edges, this is never required.
        """
        return False

    cdef inline int new_edge_label(self, object l) except -1:
        """
        Any label is ignored.
        """
        return 0

    cdef inline int free_edge_label(self, int l_int) except -1:
        """
        Free the label corresponding to ``l_int``.

        As the backend does not support labels this raises an error
        if ``l_int`` is non-trivial.
        """
        if unlikely(l_int):
            raise ValueError("backend does not support labels")

    def add_edges(self, object edges, bint directed, bint remove_loops=False):
        """
        Add edges from a list.

        INPUT:

        - ``edges`` -- an iterable of edges to be added; each edge can either be
          of the form ``(u, v)`` or ``(u, v, l)``

        - ``directed`` -- if ``False``, adds ``(v, u)`` as well as ``(u, v)``

        - ``remove_loops`` -- if ``True``, remove loops

        EXAMPLES::

            sage: D = sage.graphs.base.dense_graph.DenseGraphBackend(9)
            sage: D.add_edges([(0, 1), (2, 3), (4, 5), (5, 6)], False)
            sage: list(D.iterator_edges(range(9), True))
            [(0, 1, None),
             (2, 3, None),
             (4, 5, None),
             (5, 6, None)]
        """
        for e in edges:
            u, v = e[:2]
            if unlikely(remove_loops and u == v):
                continue
            self.add_edge(u, v, None, directed)

    def get_edge_label(self, object u, object v):
        """
        Return the edge label for ``(u, v)``.

        Always ``None``, since dense graphs do not support edge labels.

        INPUT:

        - ``u``, ``v`` -- the vertices of the edge

        EXAMPLES::

            sage: D = sage.graphs.base.dense_graph.DenseGraphBackend(9)
            sage: D.add_edges([(0, 1), (2, 3, 7), (4, 5), (5, 6)], False)
            sage: list(D.iterator_edges(range(9), True))
            [(0, 1, None),
             (2, 3, None),
             (4, 5, None),
             (5, 6, None)]
            sage: D.del_edge(0, 1, None, True)
            sage: list(D.iterator_out_edges(range(9), True))
            [(1, 0, None),
             (2, 3, None),
             (3, 2, None),
             (4, 5, None),
             (5, 4, None),
             (5, 6, None),
             (6, 5, None)]
            sage: D.get_edge_label(2, 3)
            sage: D.get_edge_label(2, 4)
            Traceback (most recent call last):
            ...
            LookupError: (2, 4) is not an edge of the graph
        """
        if not self.has_edge(u, v, None):
            raise LookupError("({0}, {1}) is not an edge of the graph".format(repr(u), repr(v)))
        return None

    def has_edge(self, object u, object v, object l):
        """
        Check whether this graph has edge ``(u, v)``.

        .. NOTE::

            The input ``l`` is for consistency with other backends.

        INPUT:

        - ``u``, ``v`` -- the vertices of the edge

        - ``l`` -- the edge label (ignored)

        EXAMPLES::

            sage: D = sage.graphs.base.dense_graph.DenseGraphBackend(9)
            sage: D.add_edges([(0, 1), (2, 3), (4, 5), (5, 6)], False)
            sage: D.has_edge(0, 1, None)
            True
        """
        cdef int u_int = self.get_vertex_checked(u)
        cdef int v_int = self.get_vertex_checked(v)
        if u_int == -1 or v_int == -1:
            return False
        return self._has_labeled_edge_unsafe(u_int, v_int, None)

    cdef inline bint _has_labeled_edge_unsafe(self, int u_int, int v_int, object l) except -1:
        """
        Return whether ``self`` has an arc specified by indices of the vertices
        and an arc label.
        """
        return 1 == self.cg().has_arc_unsafe(u_int, v_int)

    def multiple_edges(self, new):
        """
        Get/set whether or not ``self`` allows multiple edges.

        INPUT:

        - ``new`` -- boolean (to set) or ``None`` (to get)

        EXAMPLES::

            sage: import sage.graphs.base.dense_graph
            sage: G = sage.graphs.base.dense_graph.DenseGraphBackend(9)
            sage: G.multiple_edges(True)
            Traceback (most recent call last):
            ...
            NotImplementedError: dense graphs do not support multiple edges
            sage: G.multiple_edges(None)
            False
        """
        if new is None:
            return False
        if new:
            raise NotImplementedError("dense graphs do not support multiple edges")

    def set_edge_label(self, object u, object v, object l, bint directed):
        """
        Label the edge ``(u, v)`` by ``l``.

        INPUT:

        - ``u``, ``v`` -- the vertices of the edge

        - ``l`` -- the edge label

        - ``directed`` -- if ``False``, also set ``(v, u)`` with label ``l``

        EXAMPLES::

            sage: import sage.graphs.base.dense_graph
            sage: G = sage.graphs.base.dense_graph.DenseGraphBackend(9)
            sage: G.set_edge_label(1, 2, 'a', True)
            Traceback (most recent call last):
            ...
            NotImplementedError: dense graphs do not support edge labels
        """
        raise NotImplementedError("dense graphs do not support edge labels")



# sage_setup: distribution = sagemath-graphs
# sage.doctest: needs sage.combinat
r"""
`\nu`-Tamari lattice

A class of the `\nu`-Tamari lattice, and `(\delta,\nu)`-Tamari lattice (or alt
`\nu`-Tamari lattices), see [PRV2017]_ and [CC2023]_ for details.

These lattices depend on a parameter `\nu` where `\nu` is a path of North
and East steps. The alt `\nu`-Tamari lattice depends on an additional parameter
`\delta`, which is an increment vector with respect to `\nu`.

The elements are :func:`\nu-Dyck paths<sage.combinat.nu_dyck_word.NuDyckWord>`
which are weakly above `\nu`.

To use the provided functionality, you should import `\nu`-Tamari lattices by
typing::

    sage: from sage.combinat.nu_tamari_lattice import NuTamariLattice, AltNuTamariLattice

Then, ::

    sage: NuTamariLattice([1,1,1,0,0,1,1,0])
    Finite lattice containing 6 elements
    sage: NuTamariLattice([0,0,0,1,1,0,0,1])
    Finite lattice containing 40 elements
    sage: AltNuTamariLattice([0,0,0,1,1,0,0,1])
    Finite lattice containing 40 elements
    sage: AltNuTamariLattice([0,0,0,1,1,0,0,1], [0,1,0])
    Finite lattice containing 40 elements


The classical **Tamari lattices** and the **Generalized Tamari lattices** are
special cases of this construction and are also available with this poset::

    sage: NuTamariLattice([1,0,1,0,1,0])
    Finite lattice containing 5 elements

    sage: NuTamariLattice([1,0,0,1,0,0,1,0,0])
    Finite lattice containing 12 elements

.. SEEALSO::

    For more detailed information see :meth:`NuTamariLattice` and
    :meth:`AltNuTamariLattice`. For more
    information on the standard Tamari lattice see
    :meth:`sage.combinat.tamari_lattices.TamariLattice`,
    :meth:`sage.combinat.tamari_lattices.GeneralizedTamariLattice`

AUTHORS:

- Aram Dermenjian (2020-09-26): initial version

- Clément Chenevière (2024-02-01): added the alt `\nu`-Tamari lattices
"""
# ****************************************************************************
#    Copyright (C) 2020-2020 Aram Dermenjian <aram.dermenjian@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#    This code is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    General Public License for more details.
#
#  The full text of the GPL is available at:
#
#                  https://www.gnu.org/licenses/
# ****************************************************************************
from sage.combinat.nu_dyck_word import NuDyckWords, NuDyckWord
from sage.combinat.posets.lattices import LatticePoset


def NuTamariLattice(nu):
    r"""
    Return the `\nu`-Tamari lattice.

    INPUT:

    - `\nu` -- list of 0s and 1s or a string of 0s and 1s

    OUTPUT: a finite lattice

    The elements of the lattice are
    :func:`\nu-Dyck paths<sage.combinat.nu_dyck_word.NuDyckWord>` weakly above
    `\nu`.

    The usual :wikipedia:`Tamari lattice<Tamari_lattice>` is the special case
    where `\nu = (NE)^h` where `h` is the height.

    Other special cases give the `m`-Tamari lattices studied in [BMFPR]_.

    EXAMPLES::

        sage: from sage.combinat.nu_tamari_lattice import NuTamariLattice
        sage: NuTamariLattice([1,0,1,0,0,1,0])
        Finite lattice containing 7 elements
        sage: NuTamariLattice([1,0,1,0,1,0])
        Finite lattice containing 5 elements
        sage: NuTamariLattice([1,0,1,0,1,0,1,0])
        Finite lattice containing 14 elements
        sage: NuTamariLattice([1,0,1,0,1,0,0,0,1])
        Finite lattice containing 24 elements
    """
    NDW = NuDyckWords(nu)
    covers = []
    elements = []
    height = NDW[0].height()
    for ndw in NDW:
        elements.append(ndw)
        for i in range(1, height + 1):
            new_ndw = ndw.mutate(i)
            if new_ndw is not None:
                covers.append([ndw, new_ndw])
    return LatticePoset([elements, covers])


def delta_swap(p, k, delta):
    r"""
    Perform a covering move in the `(\delta,\nu)`-Tamari lattice (or alt
    `\nu`-Tamari lattice, see [CC2023]_).

    The letter at position `k` is a North step of the `\nu`-Dyck word `p`, and
    must be preceded by an East step.

    The vector `\delta = (\delta_1, \dots, \delta_n)` is an increment vector
    with respect to the path `\nu`, that is to say `\delta_i \leq \nu_i`, where
    `\nu_i` is the number of East steps following the `i`-th North step of
    `\nu`.

    INPUT:

    - ``p`` -- a `\nu`-Dyck word

    - ``k`` -- integer between `0` and ``p.length()-1``

    - ``delta`` -- list of nonnegative integers of length ``p.height()``

    OUTPUT: a `\nu`-Dyck word

    EXAMPLES::

        sage: from sage.combinat.nu_tamari_lattice import delta_swap
        sage: delta_swap(NuDyckWord('0101', '0101'), 3, delta = [1, 0])
        [0, 1, 1, 0]
        sage: delta_swap(NuDyckWord('1001110100', '0100010111'), 3, [3, 1, 0, 0, 0])
        [1, 0, 1, 1, 1, 0, 0, 1, 0, 0]
        sage: delta_swap(NuDyckWord('10100101000', '01001000110'), 2, [2, 3, 0, 1])
        [1, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0]
        sage: delta_swap(NuDyckWord('10100101000', '01001000110'), 2, [1, 1, 0, 0])
        [1, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0]


    TESTS::

        sage: delta_swap(NuDyckWord('10011101000', '01000101110'), 0, [3, 1, 0, 0, 1])
        Traceback (most recent call last):
        ...
        ValueError: there is no such covering move
        sage: delta_swap(NuDyckWord('10011101000', '01000101110'), 1, [3, 1, 0, 0, 1])
        Traceback (most recent call last):
        ...
        ValueError: there is no such covering move
        sage: delta_swap(NuDyckWord('10011101000', '01000101110'), 11, [3, 1, 0, 0, 1])
        Traceback (most recent call last):
        ...
        ValueError: the index is greater than the length of the path
    """
    if k >= p.length():
        raise ValueError("the index is greater than the length of the path")
    # if delta is None:
    #     delta = [len(_) for _ in str(p._nu).split(sep='1')[1:]]
    if k == 0 or p[k - 1] == 1:
        raise ValueError("there is no such covering move")
    found = False
    i = p[:k].count(1)
    j = k
    alt = 0
    while not found and j <= p.length() - 1:
        if p[j]:
            alt += delta[i]
            i += 1
        else:
            alt -= 1
        if alt == 0:
            found = True
        j += 1
    q = p[:k - 1] + p[k:j] + [p[k - 1]] + p[j:]
    return NuDyckWord(q, p._nu)


def AltNuTamariLattice(nu, delta=None):
    r"""
    Return the `(\delta,\nu)`-Tamari lattice (or alt `\nu`-Tamari lattice).

    For more information, see [CC2023]_.

    The path `\nu` is a path of North steps (represented as `1` s) and East
    steps (represented as `0` s).

    The vector `\delta = (\delta_1, \dots, \delta_n)` is an increment vector
    with respect to the path `\nu`, that is to say `\delta_i \leq \nu_i`, where
    `\nu_i` is the number of `0` s following the `i`-th `1` of `\nu`. If not
    provided, `\delta` is set by default to produce the classical `\nu`-Tamari
    lattice.

    INPUT:

    - `\nu` -- list of 0s and 1s or a string of 0s and 1s

    - `\delta` -- list of nonnegative integers

    OUTPUT: a finite lattice

    EXAMPLES::

        sage: from sage.combinat.nu_tamari_lattice import AltNuTamariLattice, NuTamariLattice
        sage: AltNuTamariLattice('01001', [0, 0])
        Finite lattice containing 7 elements
        sage: AltNuTamariLattice('01001', [1, 0])
        Finite lattice containing 7 elements
        sage: AltNuTamariLattice('01001') == AltNuTamariLattice('01001', [2, 0])
        True
        sage: nu = '00100100101'; P = AltNuTamariLattice(nu); Q = NuTamariLattice(nu); P == Q
        True

    TESTS::

        sage: AltNuTamariLattice('012', [0,0])
        Traceback (most recent call last):
        ...
        ValueError: nu must be a list or a string of 0s and 1s
        sage: AltNuTamariLattice([0,10,0,11], [2,0,0])
        Traceback (most recent call last):
        ...
        ValueError: nu must be a list or a string of 0s and 1s
        sage: AltNuTamariLattice('01001', [0, 1, 0])
        Traceback (most recent call last):
        ...
        ValueError: delta is not a valid increment vector
        sage: AltNuTamariLattice('0100101', [3, 0, 0])
        Traceback (most recent call last):
        ...
        ValueError: delta is not a valid increment vector

    REFERENCES:

    - [PRV2017]_

    - [CC2023]_
    """
    if not ((isinstance(nu, (list, tuple)) and all(x in [0, 1] for x in nu)) or
            (isinstance(nu, str) and all(x in ['0', '1'] for x in nu))):
        raise ValueError("nu must be a list or a string of 0s and 1s")
    nu = [int(a) for a in nu]
    # transforms nu in a sequence of 0s and 1s if it is a list
    nu = ''.join(str(a) for a in nu)
    # produces delta if delta is None, and check that delta is valid otherwise
    deltamax = [len(a) for a in nu.split(sep='1')[1:]]
    if delta is None:
        delta = deltamax
    elif len(delta) != len(deltamax) or any(delta[i] > deltamax[i] for i in range(len(delta))):
        raise ValueError("delta is not a valid increment vector")

    def covers(p):
        return [delta_swap(p, k, delta=delta) for k in range(1, p.length())
                if not p[k - 1] and p[k]]
    return LatticePoset({p: covers(p) for p in NuDyckWords(nu)},
                        check=False)



# sage_setup: distribution = sagemath-graphs
r"""
Compact structure for fast operations on less than 32 vertices

This module implements a digraph structure meant to be used in Cython in
**highly enumerative** algorithms. It can store graphs on less than
``sizeof(int)`` vertices and perform several basic operations **quickly**
(add/remove arcs, count the out-neighborhood of a set of vertices or return its
cardinality).

**Sets and integers :**

In the following code, sets are represented as integers, where the `i`-th bit is
set if element `i` belongs to the set.
"""
from cysignals.memory cimport check_allocarray, check_calloc, sig_free


cdef class FastDigraph:

    def __cinit__(self, D, vertex_list=None):
        r"""
        Constructor for ``FastDigraph``.

        If the input parameter ``D`` is a Graph, it is handled as a symmetric
        DiGraph.

        INPUT:

        - ``D`` -- a (Di)Graph

        - ``vertex_list`` -- list (default: ``None``); specifies a mapping
          between `[0..n-1]` and the set of vertices of the input (Di)Graph,
          ``list(D)`` by default

        EXAMPLES::

            sage: cython_code = [
            ....: 'from sage.graphs.graph import Graph',
            ....: 'from sage.graphs.graph_decompositions.fast_digraph cimport FastDigraph',
            ....: 'G = Graph([(0, 1), (1, 2)])',
            ....: 'cdef FastDigraph F = FastDigraph(G)',
            ....: 'cdef int i',
            ....: 'print([F.degree[i] for i in range(F.n)])']
            sage: cython(os.linesep.join(cython_code))                                  # needs sage.misc.cython
            [1, 2, 1]
        """
        if D.order() > 8*sizeof(int):
            raise OverflowError("Too many vertices. This structure can only "
                                "encode digraphs on at most "
                                "%i vertices" % (8 * sizeof(int)))

        self.n = D.order()
        self.graph = <int *>check_calloc(self.n, sizeof(int))

        cdef int i
        cdef int tmp

        # When the vertices are not consecutive integers
        if vertex_list is None:
            self.int_to_vertices = list(D)
        elif len(vertex_list) == self.n and not set(vertex_list).symmetric_difference(D):
            self.int_to_vertices = list(vertex_list)
        else:
            raise ValueError("the input vertex_list is incorrect")
        cdef dict vertices_to_int = {v: i for i, v in enumerate(self.int_to_vertices)}

        if D.is_directed():
            for u in D:
                tmp = 0
                for v in D.neighbors_out(u):
                    tmp |= 1 << vertices_to_int[v]
                self.graph[vertices_to_int[u]] = tmp
        else:
            for u in D:
                tmp = 0
                for v in D.neighbors(u):
                    tmp |= 1 << vertices_to_int[v]
                self.graph[vertices_to_int[u]] = tmp

        self.degree = <int *>check_allocarray(self.n, sizeof(int))
        for i in range(self.n):
            self.degree[i] = popcount32(self.graph[i])

    def __dealloc__(self):
        r"""
        Destructor.
        """
        sig_free(self.graph)
        sig_free(self.degree)

    def print_adjacency_matrix(self):
        r"""
        Displays the adjacency matrix of ``self``.

        EXAMPLES::

            sage: cython_code = [
            ....: 'from sage.graphs.graph import Graph',
            ....: 'from sage.graphs.graph_decompositions.fast_digraph cimport FastDigraph',
            ....: 'FastDigraph(Graph([(0, 1), (1, 2)])).print_adjacency_matrix()']
            sage: cython(os.linesep.join(cython_code))                                  # needs sage.misc.cython
            010
            101
            010
        """
        cdef int i, j
        for i in range(self.n):
            for j in range(self.n):
                print(((self.graph[i] >> j) & 1), end="")
            print("")

cdef inline int compute_out_neighborhood_cardinality(FastDigraph g, int S) noexcept:
    r"""
    Return the cardinality of `N^+(S)\S`.

    INPUT:

    - ``g`` -- a FastDigraph

    - ``S`` -- integer describing the set

    EXAMPLES::

        sage: cython_code = [
        ....: 'from sage.graphs.graph import Graph',
        ....: 'from sage.graphs.graph_decompositions.fast_digraph cimport FastDigraph',
        ....: 'from sage.graphs.graph_decompositions.fast_digraph cimport compute_out_neighborhood_cardinality',
        ....: 'cdef FastDigraph F = FastDigraph(Graph([(0, 1), (1, 2)]))',
        ....: 'cdef int i',
        ....: 'print([compute_out_neighborhood_cardinality(F, 1<<i) for i in range(F.n)])']
        sage: cython(os.linesep.join(cython_code))                                      # needs sage.misc.cython
        [1, 2, 1]
    """
    cdef int i
    cdef int tmp = 0
    for i in range(g.n):
        tmp |= g.graph[i] & (-((S >> i) & 1))

    tmp &= (~S)
    return popcount32(tmp)

cdef inline int popcount32(int i) noexcept:
    r"""
    Return the number of '1' bits in a 32-bits integer.

    If ``sizeof(int) > 4``, this function only returns the number of '1'
    bits in ``(i & ((1<<32) - 1))``.

    EXAMPLES::

        sage: cython_code = [
        ....: 'from sage.graphs.graph_decompositions.fast_digraph cimport popcount32',
        ....: 'cdef int i',
        ....: 'print([popcount32(i) for i in range(16)])']
        sage: cython(os.linesep.join(cython_code))                                      # needs sage.misc.cython
        [0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4]
    """
    i = i - ((i >> 1) & 0x55555555)
    i = (i & 0x33333333) + ((i >> 2) & 0x33333333)
    return ((i + (i >> 4) & 0x0F0F0F0F) * 0x01010101) >> 24


# If you happened to be doubting the consistency of the popcount32 function
# above, you can give the following doctest a try. It is not tested
# automatically by Sage as it takes a *LONG* time to run (around 5 minutes), but
# it would report any problem if it finds one.

def test_popcount():
    """
    Correction test for popcount32.

    EXAMPLES::

        sage: from sage.graphs.graph_decompositions.fast_digraph import test_popcount
        sage: test_popcount() # not tested
    """
    cdef int i = 1
    # While the last 32 bits of i are not equal to 0
    while (i & ((1 << 32) - 1)):
        if popcount32(i) != slow_popcount32(i):
            print("Error for i = ", str(i))
            print("Result with popcount32 : " + str(popcount32(i)))
            print("Result with slow_popcount32 : " + str(slow_popcount32(i)))
        i += 1


cdef inline int slow_popcount32(int i) noexcept:
    """
    Return the number of '1' bits in a 32-bits integer.

    If ``sizeof(int) > 4``, this function only returns the number of '1'
    bits in ``(i & ((1<<32) - 1))``.

    EXAMPLES::

        sage: cython_code = [
        ....: 'from sage.graphs.graph_decompositions.fast_digraph cimport popcount32',
        ....: 'from sage.graphs.graph_decompositions.fast_digraph cimport slow_popcount32',
        ....: 'cdef int i',
        ....: 'print(all(popcount32(i) == slow_popcount32(i) for i in range(16)))']
        sage: cython(os.linesep.join(cython_code))                                      # needs sage.misc.cython
        True
    """
    # Slow popcount for 32bits integers
    cdef int j = 0
    cdef int k

    for k in range(32):
        j += (i >> k) & 1

    return j
